/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__DEFS_H__INCLUDED__)
#define __NDD__DEFS_H__INCLUDED__

/*  64bit ǥեIO  */
#define _LARGEFILE_SOURCE   1
#define _LARGEFILE64_SOURCE 1
#define _FILE_OFFSET_BITS    64

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ndd/types.h>

#define NDD_START				\
    ndd_result __return_code = NDD_SUCCESS;

#define NDD_RETURN				\
    return __return_code;

#define NDD_ISOK				\
    (__return_code == NDD_SUCCESS)

#define NDD_IFOK				\
    if(NDD_ISOK)

#define NDD_SET_ERROR(code)						\
    {									\
	__return_code = (code);						\
	fprintf(stderr, "Error at %s(%d): code = %lu\n", __FILE__, __LINE__, __return_code); \
	abort();\
    }

#define NDD_VERIFY_RESULT(code)			\
    NDD_IFOK{					\
	__return_code = (code);			\
    }

#define NDD_VERIFY_RESULT_OR(code)		\
    if(NDD_ISOK) if((__return_code = (code)) != NDD_SUCCESS)

#define NDD_VERIFY_RESULT_AND(code)		\
    if(NDD_ISOK) if((__return_code = (code)) == NDD_SUCCESS)

#define NDD_VERIFY(cond, code)			\
    NDD_IFOK{					\
	if(!(cond)){				\
	    __return_code = code;		\
	}					\
    }

#define NDD_VERIFY_OR(cond, code)			\
    if(NDD_ISOK) if(!(cond) && ((__return_code = code), 1))

#define NDD_VERIFY_AND(cond, code)			\
    if(NDD_ISOK) if((cond) || ((__return_code = code), 1))

#define NDD_WHILE(cond)				\
    while(NDD_ISOK && (cond))

#define NDD_FOR					\
    if(NDD_ISOK) for

#define NDD_VERIFY_POINTER(data)			\
    if(NDD_ISOK && ((data) == NULL)){			\
	NDD_SET_ERROR(NDD_ERROR_INVALID_POINTER);	\
    }

#define NDD_UNUSED(data)			\
    { data = data; }

/*  顼åϤ롩  */
extern FILE     *ndd_debug_stream;
extern ndd_bool  ndd_is_verbose;

#endif  /*  #if !defined(__NDD__DEFS_H__INCLUDED__)  */

/*  end of file  */
