/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__FFD_H__INCLUDED__)
#define __NDD__FFD_H__INCLUDED__

#include <ndd/defs.h>

struct __ndd_ffd_data
{
    ndd_float8 **frame_vector;  /*  vector_data[frame_dimension][mf]  */
};

struct __ndd_ffd
{
    ndd_file_header *header;
    ndd_ffs_data    *ffs;
    ndd_ffd_data    *ffd;
};

extern ndd_result ndd_ffd_create (ndd_ffd **ffd);
extern ndd_result ndd_ffd_destroy(ndd_ffd **ffd);

extern ndd_result ndd_ffd_copy(ndd_ffd *dst_ffd, const ndd_ffd *src_ffd);

extern ndd_result ndd_ffd_read_from_file(ndd_ffd       *ffd, FILE *fp);
extern ndd_result ndd_ffd_write_to_file (const ndd_ffd *ffd, FILE *fp);
extern ndd_result ndd_ffd_print_to_file (const ndd_ffd *ffd, FILE *fp);

extern ndd_result ndd_ffd_cut(ndd_ffd *dst_ffd, const ndd_ffd *src_ffd, ndd_uint32 frame_offset, ndd_uint32 frame_count);
extern ndd_result ndd_ffd_concatenate(ndd_ffd *dst_ffd, const ndd_ffd *src_ffd_1, const ndd_ffd *src_ffd_2);


extern ndd_result ndd_ffd_data_create (ndd_ffd_data **ffd_data);
extern ndd_result ndd_ffd_data_destroy(ndd_ffd_data **ffd_data, const ndd_file_header *header);

/*  Note: ffd_data ǡä֤ ndd_ffd_data_read_from_file Ƥ֤ȡ꡼  */
extern ndd_result ndd_ffd_data_read_from_file(ndd_ffd_data       *ffd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_ffd_data_write_to_file (const ndd_ffd_data *ffd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_ffd_data_print_to_file (const ndd_ffd_data *ffd_data, const ndd_file_header *header, FILE *fp);

/*  Note: dst_ffd_data ǡä֤ ndd_ffd_data_copy Ƥ֤ȡ꡼  */
extern ndd_result ndd_ffd_data_copy(ndd_ffd_data *dst_ffd_data, const ndd_ffd_data *src_ffd_data, const ndd_file_header *header);

extern ndd_result ndd_ffd_compare_format(const ndd_ffd *ffd1, const ndd_ffd *ffd2, ndd_bool *is_same);
extern ndd_result ndd_ffd_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same);

#endif  /*  #if !defined(__NDD__FFD_H__INCLUDED__)  */

/*  end of file  */
