/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__FILE_H__INCLUDED__)
#define __NDD__FILE_H__INCLUDED__

#include <ndd/defs.h>

#define NDD_FILE_MARKER_FVD   0x44564621  /*  "!FVD"  */
#define NDD_FILE_MARKER_FFS   0x53464621  /*  "!FFS"  */
#define NDD_FILE_MARKER_FFD   0x44464621  /*  "!FFD"  */
#define NDD_FILE_MARKER_SFS   0x53465321  /*  "!SFS"  */
#define NDD_FILE_MARKER_SFD   0x44465321  /*  "!SFD"  */

#define NDD_FILE_VERSION_FVD  0x01000000
#define NDD_FILE_VERSION_FFS  0x01000000
#define NDD_FILE_VERSION_FFD  0x01000000
#define NDD_FILE_VERSION_SFS  0x01000000
#define NDD_FILE_VERSION_SFD  0x01000000

struct __ndd_file_header
{
    ndd_uint32 marker;
    ndd_uint32 version;
    ndd_uint32 reserved1;
    ndd_uint32 reserved2;

    ndd_uint32 screen_width;   /*  MPEG  */
    ndd_uint32 screen_height;  /*  MPEG  */
    ndd_uint32 region_left;    /*  FVD ˥桼  */
    ndd_uint32 region_top;     /*  FVD ˥桼  */

    ndd_uint32 region_width;     /*  FVD ˥桼  */
    ndd_uint32 region_height;    /*  FVD ˥桼  */
    ndd_uint32 block_width;      /*  FVD ˥桼  */
    ndd_uint32 block_height;     /*  FVD ˥桼  */

    ndd_uint32 color_resolution;    /*  FVD ˥桼  */
    ndd_uint32 reserved3;
    ndd_uint32 reserved4;
    ndd_uint32 reserved5;

    ndd_float8 frames_per_second;    /*  MPEG   */
    ndd_uint32 total_frame_count;    /*  MPEG   */
    ndd_uint32 reserved6;

    ndd_uint32 frame_offset;    /*  FVD ˥桼  */
    ndd_uint32 frame_count;     /*  FVD ˥桼  */
    ndd_uint32 reserved7;
    ndd_uint32 reserved8;

    ndd_float8 frame_proportion;     /*  FFS ˥桼  */
    ndd_uint32 frame_dimension;      /*  FFS ˥桼  */
    ndd_uint32 frame_sample_count;   /*  FFS ˥桼  */

    ndd_float8 segments_per_second;  /*  SFS ˥桼  */
    ndd_uint32 segment_size;         /*  SFS ˥桼  */
    ndd_uint32 total_segment_count;  /*  SFS ˷׻  */

    ndd_uint32 segment_offset;  /*  SFD ˥桼  */
    ndd_uint32 segment_count;   /*  SFD ˥桼  */
    ndd_uint32 reserved9;
    ndd_uint32 reserved10;

    ndd_float8 segment_proportion;    /*  SFS ˥桼  */
    ndd_uint32 segment_dimension;     /*  SFS ˥桼  */
    ndd_uint32 segment_sample_count;  /*  SFS ˥桼  */

    ndd_float8 sfd_search_threshold;  /*  õ˥桼  */
    ndd_float8 ffd_search_threshold;   /*  õ˥桼  */

    ndd_float8 fvd_search_threshold;  /*  õ˥桼  */
    ndd_uint32 minimum_segment_size;  /*  õ˥桼  */
    ndd_uint32 reserved13;
    
};

extern ndd_result ndd_file_header_create (ndd_file_header **file_header);
extern ndd_result ndd_file_header_destroy(ndd_file_header **file);

extern ndd_result ndd_file_header_read_from_file(ndd_file_header       *file_header, FILE *fp);
extern ndd_result ndd_file_header_write_to_file (const ndd_file_header *file_header, FILE *fp);
extern ndd_result ndd_file_header_print_to_file (const ndd_file_header *file_header, FILE *fp);

extern ndd_result ndd_file_header_copy(ndd_file_header       *dst_file_header,
				       const ndd_file_header *src_file_header);

extern ndd_result ndd_file_header_get_elements_per_frame(const ndd_file_header *file_header, ndd_uint32 *elements_per_frame);
extern ndd_result ndd_file_header_get_elements_per_segment(const ndd_file_header *file_header, ndd_uint32 *elements_per_segment);

extern ndd_result ndd_file_header_get_frame_count       (const ndd_file_header *file_header, ndd_uint32 *frame_count);
extern ndd_result ndd_file_header_get_segment_count       (const ndd_file_header *file_header, ndd_uint32 *segment_count);

/*  ҤȤĤΥȤĹˡʬΥȤγ¸ߤ뤫 */
extern ndd_result ndd_file_header_get_segments_per_segment(const ndd_file_header *file_header, ndd_uint32 *segment_count);

#endif  /*  #if !defined(__NDD__FILE_H__INCLUDED__)  */

/*  end of file  */
