/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__FVD_H__INCLUDED__)
#define __NDD__FVD_H__INCLUDED__

#include <ndd/defs.h>

#define NDD_FVD_CACHE_COUNT 2     /*  å粿Ѱդ뤫  */
#define NDD_FVD_CACHE_SIZE  4096  /*  粿ե졼ɤߤ뤫  */

typedef enum __ndd_file_mode
{
    ndd_file_mode_unknown = 0,
    ndd_file_mode_read,
    ndd_file_mode_write,
} ndd_file_mode;

struct __ndd_fvd_data_cache
{
    ndd_uint32  frame_offset; /*  frame_data ˥ɤƤǡΥե졼  */
    ndd_uint32  frame_count;
    ndd_float8 *frame_data[NDD_FVD_CACHE_SIZE];
};

struct __ndd_fvd_data
{
    ndd_uint32          cache_count;
    ndd_fvd_data_cache *caches[NDD_FVD_CACHE_COUNT];
};

struct __ndd_fvd
{
    FILE            *file;
    ndd_file_mode    file_mode;
    off_t            frame_data_offset;  /* եǤΥե졼ǡƬ  */

    ndd_file_header *header;
    ndd_fvd_data    *fvd;
};

extern ndd_result ndd_fvd_create (ndd_fvd **fvd);
extern ndd_result ndd_fvd_destroy(ndd_fvd **fvd);

//extern ndd_result ndd_fvd_copy(ndd_fvd *dst_fvd, const ndd_fvd *src_fvd);

/*
extern ndd_result ndd_fvd_read_from_file(ndd_fvd       *fvd, FILE *fp);
extern ndd_result ndd_fvd_write_to_file (const ndd_fvd *fvd, FILE *fp);
*/
extern ndd_result ndd_fvd_print_to_file (const ndd_fvd *fvd, FILE *fp);

extern ndd_result ndd_fvd_cut(ndd_fvd *dst_fvd, ndd_fvd *src_fvd, ndd_uint32 frame_offset, ndd_uint32 frame_count);
/*
extern ndd_result ndd_fvd_concatenate(ndd_fvd *dst_fvd, const ndd_fvd *src_fvd_1, const ndd_fvd *src_fvd_2);
*/
extern ndd_result ndd_fvd_append(ndd_fvd *dst_fvd, ndd_fvd *src_fvd);

extern ndd_result ndd_fvd_data_create (ndd_fvd_data **fvd_data);
extern ndd_result ndd_fvd_data_destroy(ndd_fvd_data **fvd_data, const ndd_file_header *header);

/*  Note: fvd_data ǡä֤ ndd_fvd_data_read_from_file Ƥ֤ȡ꡼  */
/*
extern ndd_result ndd_fvd_data_read_from_file(ndd_fvd_data       *fvd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_fvd_data_write_to_file (const ndd_fvd_data *fvd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_fvd_data_print_to_file (const ndd_fvd_data *fvd_data, const ndd_file_header *header, FILE *fp);
*/

/*  Note: dst_fvd_data ǡä֤ ndd_fvd_data_copy Ƥ֤ȡ꡼  */
//extern ndd_result ndd_fvd_data_copy(ndd_fvd_data *dst_fvd_data, const ndd_fvd_data *src_fvd_data, const ndd_file_header *header);

extern ndd_result ndd_fvd_compare_format       (const ndd_fvd *fvd1, const ndd_fvd *fvd2, ndd_bool *is_same);
extern ndd_result ndd_fvd_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same);



extern ndd_result ndd_fvd_assign_file(ndd_fvd *fvd, const char *filename, ndd_file_mode file_mode);
extern ndd_result ndd_fvd_read_frame_data(ndd_fvd *fvd, ndd_uint32 frame_index,
					  ndd_float8 *frame_data, ndd_uint32 frame_data_count);
extern ndd_result ndd_fvd_read_frame_data_direction(ndd_fvd *fvd, ndd_uint32 frame_index,
						    ndd_float8 *frame_data, ndd_uint32 frame_data_count,
						    ndd_bool forward);
extern ndd_result ndd_fvd_write_frame_data(ndd_fvd *fvd, ndd_uint32 frame_index,
					   const ndd_float8 *frame_data, ndd_uint32 frame_data_count);

#endif  /*  #if !defined(__NDD__FVD_H__INCLUDED__)  */

/*  end of file  */
