/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__SFD_H__INCLUDED__)
#define __NDD__SFD_H__INCLUDED__

#include <ndd/defs.h>

struct __ndd_sfd_data
{
    ndd_float8 **segment_vector;  /*  vector_data[frame_dimension][mf]  */
};

struct __ndd_sfd
{
    ndd_file_header *header;
    ndd_ffs_data    *ffs;
    ndd_sfs_data    *sfs;
    ndd_sfd_data    *sfd;
};

extern ndd_result ndd_sfd_create (ndd_sfd **sfd);
extern ndd_result ndd_sfd_destroy(ndd_sfd **sfd);

extern ndd_result ndd_sfd_copy(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd);

extern ndd_result ndd_sfd_read_from_file(ndd_sfd       *sfd, FILE *fp);
extern ndd_result ndd_sfd_write_to_file (const ndd_sfd *sfd, FILE *fp);
extern ndd_result ndd_sfd_print_to_file (const ndd_sfd *sfd, FILE *fp);

extern ndd_result ndd_sfd_cut(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd, ndd_uint32 frame_offset, ndd_uint32 frame_count);
extern ndd_result ndd_sfd_concatenate(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd_1, const ndd_sfd *src_sfd_2);


extern ndd_result ndd_sfd_data_create (ndd_sfd_data **sfd_data);
extern ndd_result ndd_sfd_data_destroy(ndd_sfd_data **sfd_data, const ndd_file_header *header);

/*  Note: sfd_data ǡä֤ ndd_sfd_data_read_from_file Ƥ֤ȡ꡼  */
extern ndd_result ndd_sfd_data_read_from_file(ndd_sfd_data       *sfd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_sfd_data_write_to_file (const ndd_sfd_data *sfd_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_sfd_data_print_to_file (const ndd_sfd_data *sfd_data, const ndd_file_header *header, FILE *fp);

/*  Note: dst_sfd_data ǡä֤ ndd_sfd_data_copy Ƥ֤ȡ꡼  */
extern ndd_result ndd_sfd_data_copy(ndd_sfd_data *dst_sfd_data, const ndd_sfd_data *src_sfd_data, const ndd_file_header *header);

extern ndd_result ndd_sfd_compare_format(const ndd_sfd *sfd1, const ndd_sfd *sfd2, ndd_bool *is_same);
extern ndd_result ndd_sfd_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same);

#endif  /*  #if !defined(__NDD__SFD_H__INCLUDED__)  */

/*  end of file  */
