/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#if !defined(__NDD__SFS_H__INCLUDED__)
#define __NDD__SFS_H__INCLUDED__

#include <ndd/defs.h>

struct __ndd_sfs_data
{
    ndd_float8 **selector_vector;  /*  vector_data[frame_dimension][mf]  */
};

struct __ndd_sfs
{
    ndd_file_header *header;
    ndd_ffs_data    *ffs;
    ndd_sfs_data    *sfs;
};

extern ndd_result ndd_sfs_create (ndd_sfs **sfs);
extern ndd_result ndd_sfs_destroy(ndd_sfs **sfs);

extern ndd_result ndd_sfs_read_from_file(ndd_sfs       *sfs, FILE *fp);
extern ndd_result ndd_sfs_write_to_file (const ndd_sfs *sfs, FILE *fp);
extern ndd_result ndd_sfs_print_to_file (const ndd_sfs *sfs, FILE *fp);


extern ndd_result ndd_sfs_data_create (ndd_sfs_data **sfs_data);
extern ndd_result ndd_sfs_data_destroy(ndd_sfs_data **sfs_data, const ndd_file_header *header);

/*  Note: sfs_data ǡä֤ ndd_sfs_data_read_from_file Ƥ֤ȡ꡼  */
extern ndd_result ndd_sfs_data_read_from_file(ndd_sfs_data       *sfs_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_sfs_data_write_to_file (const ndd_sfs_data *sfs_data, const ndd_file_header *header, FILE *fp);
extern ndd_result ndd_sfs_data_print_to_file (const ndd_sfs_data *sfs_data, const ndd_file_header *header, FILE *fp);

/*  Note: dst_sfs_data ǡä֤ ndd_sfs_data_copy Ƥ֤ȡ꡼  */
extern ndd_result ndd_sfs_data_copy(ndd_sfs_data *dst_sfs_data, const ndd_sfs_data *src_sfs_data, const ndd_file_header *header);

extern ndd_result ndd_sfs_compare_format(const ndd_sfs *sfs1, const ndd_sfs *sfs2, ndd_bool *is_same);
extern ndd_result ndd_sfs_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same);
extern ndd_result ndd_sfs_data_compare  (const ndd_sfs_data *sfs_data1, const ndd_file_header *header1,
					 const ndd_sfs_data *sfs_data2, const ndd_file_header *header2, ndd_bool *is_same);

#endif  /*  #if !defined(__NDD__SFS_H__INCLUDED__)  */

/*  end of file  */
