/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/file.h>
#include <ndd/error.h>
#include <ndd/fvd.h>
#include <ndd/ffs.h>
#include <ndd/ffd.h>
#include <ndd/sfs.h>
#include <ndd/sfd.h>
#include <stdlib.h>
#include <math.h>

static ndd_result
do_printfile(const char *filename, FILE *stream)
{
    NDD_START;

    FILE *fp = NULL;
    NDD_VERIFY_OR((fp = fopen(filename, "r")) != NULL, NDD_ERROR_FILE_OPEN_FAILED){
	fprintf(stderr, "Failed to open file <%s>\n", filename);
    }

    ndd_file_header *file_header = NULL;
    NDD_VERIFY_RESULT_OR(ndd_file_header_create(&file_header)){
	fprintf(stderr, "Failed to create file header data\n");
    }

    NDD_VERIFY_RESULT_OR(ndd_file_header_read_from_file(file_header, fp)){
	fprintf(stderr, "Failed to read file header from file <%s>\n", filename);
    }

    NDD_IFOK{
	fprintf(stream, "NDD File '%s'\n", filename);
    }

    NDD_IFOK{
	rewind(fp);
	switch(file_header->marker){
	    /*
	case NDD_FILE_MARKER_FVD:
	    {
		ndd_fvd *fvd = NULL;
		NDD_VERIFY_RESULT(ndd_fvd_create(&fvd));
		NDD_VERIFY_RESULT(ndd_fvd_read_from_file(fvd, fp));
		NDD_VERIFY_RESULT(ndd_fvd_print_to_file(fvd, stream));
		if(fvd != NULL){ ndd_fvd_destroy(&fvd); }
	    }break;
	    */
	case NDD_FILE_MARKER_FFS:
	    {
		ndd_ffs *ffs = NULL;
		NDD_VERIFY_RESULT(ndd_ffs_create(&ffs));
		NDD_VERIFY_RESULT(ndd_ffs_read_from_file(ffs, fp));
		NDD_VERIFY_RESULT(ndd_ffs_print_to_file(ffs, stream));
		if(ffs != NULL){ ndd_ffs_destroy(&ffs); }
	    }break;
	case NDD_FILE_MARKER_FFD:
	    {
		ndd_ffd *ffd = NULL;
		NDD_VERIFY_RESULT(ndd_ffd_create(&ffd));
		NDD_VERIFY_RESULT(ndd_ffd_read_from_file(ffd, fp));
		NDD_VERIFY_RESULT(ndd_ffd_print_to_file(ffd, stream));
		if(ffd != NULL){ ndd_ffd_destroy(&ffd); }
	    }break;
	case NDD_FILE_MARKER_SFS:
	    {
		ndd_sfs *sfs = NULL;
		NDD_VERIFY_RESULT(ndd_sfs_create(&sfs));
		NDD_VERIFY_RESULT(ndd_sfs_read_from_file(sfs, fp));
		NDD_VERIFY_RESULT(ndd_sfs_print_to_file(sfs, stream));
		if(sfs != NULL){ ndd_sfs_destroy(&sfs); }
	    }break;
	case NDD_FILE_MARKER_SFD:
	    {
		ndd_sfd *sfd = NULL;
		NDD_VERIFY_RESULT(ndd_sfd_create(&sfd));
		NDD_VERIFY_RESULT(ndd_sfd_read_from_file(sfd, fp));
		NDD_VERIFY_RESULT(ndd_sfd_print_to_file(sfd, stream));
		if(sfd != NULL){ ndd_sfd_destroy(&sfd); }
	    }break;
	default:
	    NDD_VERIFY_RESULT(ndd_file_header_print_to_file(file_header, stream));
	    break;
	}
    }

    if(file_header){ ndd_file_header_destroy(&file_header); }
    if(fp){ fclose(fp); }

    NDD_RETURN;
}

int main(int argc, char *argv[])
{
    if(argc < 2){
	fprintf(stderr, "Usage: %s <filename>\n", argv[0]);
	return EXIT_FAILURE;
    }

    ndd_result result = do_printfile(argv[1], stdout);

    if(result == NDD_SUCCESS){
	return EXIT_SUCCESS;
    }else{
	fprintf(stderr, "Error code = %lu\n", result);
	return EXIT_FAILURE;
    }
}

/*  end of file  */
