/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/file.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/defaults.h>

#include <ctype.h>

ndd_result
ndd_file_header_create(ndd_file_header **file_header)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);

    /*  ΰ  */
    ndd_file_header *new_file_header = NULL;
    NDD_VERIFY_RESULT(ndd_malloc(&new_file_header, sizeof(ndd_file_header)));

    /*  ѿν  */
    NDD_IFOK{
	new_file_header->marker                 = 0;
	new_file_header->version                = 0;
	new_file_header->reserved1              = 0;
	new_file_header->reserved2              = 0;
	
	new_file_header->screen_width           = 0;
	new_file_header->screen_height          = 0;
	new_file_header->region_left            = 0;
	new_file_header->region_top             = 0;
	
	new_file_header->region_width           = 0;
	new_file_header->region_height          = 0;
	new_file_header->block_width            = DEFAULT_BLOCK_WIDTH;
	new_file_header->block_height           = DEFAULT_BLOCK_HEIGHT;
	
	new_file_header->color_resolution       = DEFAULT_COLOR_RESOLUTION;
	new_file_header->reserved3              = 0;
	new_file_header->reserved4              = 0;
	new_file_header->reserved5              = 0;
	
	new_file_header->frames_per_second      = 10;
	new_file_header->total_frame_count      = 0;
	new_file_header->reserved6              = 0;

	new_file_header->frame_offset           = 0;
	new_file_header->frame_count            = 0;
	new_file_header->reserved7              = 0;
	new_file_header->reserved8              = 0;
		
	new_file_header->frame_proportion       = 0;//DEFAULT_FRAME_PROPORTION;
	new_file_header->frame_dimension        = 0;//DEFAULT_FRAME_DIMENSION;
	new_file_header->frame_sample_count     = DEFAULT_FRAME_SAMPLE_COUNT;
	
	new_file_header->segments_per_second    = 10;
	new_file_header->total_segment_count    = 0;
	new_file_header->segment_size           = 150;
	
	new_file_header->segment_offset         = 150;
	new_file_header->segment_count          = 0;
	new_file_header->reserved9              = 0;
	new_file_header->reserved10             = 0;

	new_file_header->segment_proportion     = 0;//DEFAULT_SEGMENT_PROPORTION;
	new_file_header->segment_dimension      = 0;//DEFAULT_SEGMENT_DIMENSION;
	new_file_header->segment_sample_count   = DEFAULT_SEGMENT_SAMPLE_COUNT;
	
	new_file_header->sfd_search_threshold   = DEFAULT_SFD_THRESHOLD;
	new_file_header->ffd_search_threshold   = DEFAULT_FFD_THRESHOLD;

	new_file_header->fvd_search_threshold   = DEFAULT_FVD_THRESHOLD;
	new_file_header->minimum_segment_size   = DEFAULT_MINIMUM_SEGMENT_SIZE;
  	new_file_header->reserved13             = 0;
  }

    /*  ̤Ǽ  */
    NDD_IFOK{
	*file_header = new_file_header;
	new_file_header = NULL;
    }

    /*  פʥǡդ  */
    if(new_file_header != NULL){
	ndd_free(&new_file_header);
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_destroy(ndd_file_header **file_header)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);

    /*  ɬפѿ  */
    NDD_IFOK{
	/*  ʤ  */
    }

    /*  ΰ  */
    NDD_VERIFY_RESULT(ndd_free(file_header));

    NDD_RETURN;
}

ndd_result
ndd_file_header_get_elements_per_frame(const ndd_file_header *file_header,
				       ndd_uint32            *elements_per_frame)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(elements_per_frame);

    NDD_IFOK{
	*elements_per_frame =
	    file_header->screen_width  / file_header->block_width  *
	    file_header->screen_height / file_header->block_height * 3;
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_get_elements_per_segment(const ndd_file_header *file_header,
				       ndd_uint32            *elements_per_segment)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(elements_per_segment);

    NDD_IFOK{
	*elements_per_segment =
	    file_header->segment_size * file_header->frame_dimension;
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_get_frame_count(const ndd_file_header *file_header,
				ndd_uint32            *frame_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(frame_count);

    NDD_IFOK{
	*frame_count = file_header->frame_count;
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_get_segment_count(const ndd_file_header *file_header,
				ndd_uint32            *segment_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(segment_count);

    NDD_IFOK{
	*segment_count = file_header->segment_count;
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_get_segments_per_segment(const ndd_file_header *file_header,
					 ndd_uint32            *segment_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(segment_count);

    NDD_IFOK{
	*segment_count = file_header->segments_per_second * file_header->segment_size / file_header->frames_per_second;
    }

    NDD_RETURN;
}

#define VERIFY_FREAD(address, size, count, fp)				\
    NDD_VERIFY(fread(address, size, count, fp) == count, NDD_ERROR_FILE_READ_FAILED)

ndd_result
ndd_file_header_read_from_file(ndd_file_header *file_header,
			       FILE            *fp)
{
    NDD_START;
    VERIFY_FREAD(&file_header->marker,                 sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->version,                sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved1,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved2,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FREAD(&file_header->screen_width,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->screen_height,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->region_left,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->region_top,             sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FREAD(&file_header->region_width,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->region_height,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->block_width,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->block_height,           sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FREAD(&file_header->color_resolution,       sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved3,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved4,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved5,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FREAD(&file_header->frames_per_second,      sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->total_frame_count,      sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved6,              sizeof(ndd_uint32), 1, fp);
   
    VERIFY_FREAD(&file_header->frame_offset,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->frame_count,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved7,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved8,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FREAD(&file_header->frame_proportion,       sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->frame_dimension,        sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->frame_sample_count,     sizeof(ndd_uint32), 1, fp);

    VERIFY_FREAD(&file_header->segments_per_second,     sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->total_segment_count,    sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->segment_size,           sizeof(ndd_uint32), 1, fp);

    VERIFY_FREAD(&file_header->segment_offset,         sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->segment_count,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved9,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved10,             sizeof(ndd_uint32), 1, fp);

    VERIFY_FREAD(&file_header->segment_proportion,     sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->segment_dimension,      sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->segment_sample_count,   sizeof(ndd_uint32), 1, fp);

    VERIFY_FREAD(&file_header->sfd_search_threshold,   sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->ffd_search_threshold,   sizeof(ndd_float8), 1, fp);

    VERIFY_FREAD(&file_header->fvd_search_threshold,   sizeof(ndd_float8), 1, fp);
    VERIFY_FREAD(&file_header->minimum_segment_size,   sizeof(ndd_uint32), 1, fp);
    VERIFY_FREAD(&file_header->reserved13,             sizeof(ndd_uint32), 1, fp);

    NDD_RETURN;
}


#define VERIFY_FWRITE(address, size, count, fp)				\
    NDD_VERIFY(fwrite(address, size, count, fp) == count, NDD_ERROR_FILE_WRITE_FAILED)

ndd_result
ndd_file_header_write_to_file(const ndd_file_header *file_header,
			      FILE                  *fp)
{
    NDD_START;
    VERIFY_FWRITE(&file_header->marker,                 sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->version,                sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved1,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved2,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FWRITE(&file_header->screen_width,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->screen_height,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->region_left,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->region_top,             sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FWRITE(&file_header->region_width,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->region_height,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->block_width,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->block_height,           sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FWRITE(&file_header->color_resolution,       sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved3,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved4,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved5,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FWRITE(&file_header->frames_per_second,      sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->total_frame_count,      sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved6,              sizeof(ndd_uint32), 1, fp);
   
    VERIFY_FWRITE(&file_header->frame_offset,           sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->frame_count,            sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved7,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved8,              sizeof(ndd_uint32), 1, fp);
    
    VERIFY_FWRITE(&file_header->frame_proportion,       sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->frame_dimension,        sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->frame_sample_count,     sizeof(ndd_uint32), 1, fp);

    VERIFY_FWRITE(&file_header->segments_per_second,    sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->total_segment_count,    sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->segment_size,           sizeof(ndd_uint32), 1, fp);

    VERIFY_FWRITE(&file_header->segment_offset,         sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->segment_count,          sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved9,              sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved10,             sizeof(ndd_uint32), 1, fp);

    VERIFY_FWRITE(&file_header->segment_proportion,     sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->segment_dimension,      sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->segment_sample_count,   sizeof(ndd_uint32), 1, fp);

    VERIFY_FWRITE(&file_header->sfd_search_threshold,   sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->ffd_search_threshold,   sizeof(ndd_float8), 1, fp);

    VERIFY_FWRITE(&file_header->fvd_search_threshold,   sizeof(ndd_float8), 1, fp);
    VERIFY_FWRITE(&file_header->minimum_segment_size,   sizeof(ndd_uint32), 1, fp);
    VERIFY_FWRITE(&file_header->reserved13,             sizeof(ndd_uint32), 1, fp);
    
       NDD_RETURN;
}

ndd_result
ndd_file_header_print_to_file(const ndd_file_header *file_header, FILE *stream)
{
    NDD_START;
    NDD_VERIFY_POINTER(file_header);
    NDD_VERIFY_POINTER(stream);

    NDD_IFOK{
	ndd_uint8 marker_ch[4] = {
	    (ndd_uint8)((file_header->marker & 0x000000FF) >>  0),
	    (ndd_uint8)((file_header->marker & 0x0000FF00) >>  8),
	    (ndd_uint8)((file_header->marker & 0x00FF0000) >> 16),
	    (ndd_uint8)((file_header->marker & 0xFF000000) >> 24),
	};
	fprintf(stream, "FILE Header\n");
	fprintf(stream, "  marker                 = 0x%08lX (%c%c%c%c)\n",
		file_header->marker,
		isprint(marker_ch[0]) ? marker_ch[0] : '?',
		isprint(marker_ch[1]) ? marker_ch[1] : '?',
		isprint(marker_ch[2]) ? marker_ch[2] : '?',
		isprint(marker_ch[3]) ? marker_ch[3] : '?');

	fprintf(stream, "  version                = %lu.%lu.%lu.%lu\n",
		(file_header->version >> 24) & 0xFF,
		(file_header->version >> 16) & 0xFF,
		(file_header->version >>  8) & 0xFF,
		(file_header->version >>  0) & 0xFF);

	fprintf(stream, "  screen_size            = [X:%4lu, Y:%4lu]\n", file_header->screen_width, file_header->screen_height);
	fprintf(stream, "  region                 = [X:%4lu, Y:%4lu] + [W:%4lu, H:%4lu]\n",
		file_header->region_left, file_header->region_top,
		file_header->region_width, file_header->region_height);

	fprintf(stream, "  block_size             = [W:%4lu, H:%4lu]\n", file_header->block_width, file_header->block_height);
	fprintf(stream, "  color_resolution       = %lu\n", file_header->color_resolution);
	
	fprintf(stream, "  frames_per_second      = %.3f\n", file_header->frames_per_second);
	fprintf(stream, "  total_frame_count      = %lu\n", file_header->total_frame_count);

	fprintf(stream, "  frame_offset           = %lu\n", file_header->frame_offset);
	fprintf(stream, "  frame_count            = %lu\n", file_header->frame_count);
	
	fprintf(stream, "  frame_proportion       = %.3f\n", file_header->frame_proportion);
	fprintf(stream, "  frame_dimension        = %lu\n", file_header->frame_dimension);
	fprintf(stream, "  frame_sample_count     = %lu\n", file_header->frame_sample_count);

	fprintf(stream, "  segments_per_second    = %.3f\n", file_header->segments_per_second);
	fprintf(stream, "  total_segment_count    = %lu\n", file_header->total_segment_count);
	fprintf(stream, "  segment_size           = %lu\n", file_header->segment_size);
	
	fprintf(stream, "  segment_offset         = %lu\n", file_header->segment_offset);
	fprintf(stream, "  segment_count          = %lu\n", file_header->segment_count);
	
	fprintf(stream, "  segment_proportion     = %.3f\n", file_header->segment_proportion);
	fprintf(stream, "  segment_dimension      = %lu\n", file_header->segment_dimension);
	fprintf(stream, "  segment_sample_count   = %lu\n", file_header->segment_sample_count);
	
	fprintf(stream, "  sfd_search_threshold   = %0.3f\n", file_header->sfd_search_threshold);
	fprintf(stream, "  ffd_search_threshold   = %0.3f\n", file_header->ffd_search_threshold);
	
	fprintf(stream, "  fvd_search_threshold   = %0.3f\n", file_header->fvd_search_threshold);
	fprintf(stream, "  minimum_segment_size   = %lu\n", file_header->minimum_segment_size);
	
    }

    NDD_RETURN;
}

ndd_result
ndd_file_header_copy(ndd_file_header       *dst_file_header,
		     const ndd_file_header *src_file_header)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_file_header);
    NDD_VERIFY_POINTER(src_file_header);

    NDD_IFOK{
	*dst_file_header = *src_file_header;
    }

    NDD_RETURN;
}

/*  end of file  */
