/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/fvd.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/file.h>

#include <string.h>

ndd_result
ndd_fvd_create(ndd_fvd **fvd)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(fvd);

    /*  ΰ  */
    ndd_fvd *new_fvd = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_fvd, sizeof(ndd_fvd)));

    /*  ѿν  */
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_fvd->header));
    NDD_VERIFY_RESULT(ndd_fvd_data_create   (&new_fvd->fvd));

    /*  ̤Ǽ  */
    NDD_IFOK{
	*fvd = new_fvd;
	new_fvd = NULL;
    }

    /*  פʥǡդ  */
    if(new_fvd != NULL){
	ndd_fvd_destroy(&new_fvd);
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_destroy(ndd_fvd **fvd)
{
    NDD_START;
    NDD_VERIFY_POINTER(fvd);

    ndd_fvd *old_fvd;
    NDD_IFOK{
	old_fvd = *fvd;
    }

    NDD_VERIFY_RESULT(ndd_fvd_data_destroy   (&old_fvd->fvd, old_fvd->header));
    NDD_VERIFY_RESULT(ndd_file_header_destroy(&old_fvd->header));
    NDD_VERIFY_RESULT(ndd_free(&old_fvd));
    
    NDD_IFOK{
	*fvd = NULL;
    }

    NDD_RETURN;
}

//ndd_result
//ndd_fvd_copy(ndd_fvd *dst_fvd, const ndd_fvd *src_fvd)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(dst_fvd);
//    NDD_VERIFY_POINTER(src_fvd);
//
//    /*  إå򥳥ԡ  */
//    ndd_file_header *new_header = NULL;
//    NDD_VERIFY_RESULT(ndd_file_header_create(&new_header));
//    NDD_VERIFY_RESULT(ndd_file_header_copy(new_header, src_fvd->header));
//
//    /*  FVD ǡ򥳥ԡ  */
//    ndd_fvd_data *new_fvd_data = NULL;
//    NDD_VERIFY_RESULT(ndd_fvd_data_create(&new_fvd_data));
//    NDD_VERIFY_RESULT(ndd_fvd_data_copy(new_fvd_data, src_fvd->fvd, src_fvd->header));
//
//    /*  ǡǼ  */
//    NDD_IFOK{
//	if(dst_fvd->fvd != NULL){
//	    NDD_VERIFY_RESULT(ndd_fvd_data_destroy(&dst_fvd->fvd, dst_fvd->header));
//	}
//	if(dst_fvd->header != NULL){
//	    NDD_VERIFY_RESULT(ndd_file_header_destroy(&dst_fvd->header));
//	}
//    }
//    NDD_IFOK{
//	dst_fvd->header = new_header;
//	dst_fvd->fvd    = new_fvd_data;
//	new_header   = NULL;
//	new_fvd_data = NULL;
//    }
//
//    if(new_fvd_data != NULL){  ndd_fvd_data_destroy   (&new_fvd_data, new_header);  }
//    if(new_header   != NULL){  ndd_file_header_destroy(&new_header);    }
//
//    NDD_RETURN;
//}

//ndd_result
//ndd_fvd_read_from_file(ndd_fvd *fvd, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd);
//    NDD_VERIFY_POINTER(fp);
//
//    /*  إåɤ߹  */
//    ndd_file_header *new_header = NULL;
//    NDD_VERIFY_RESULT(ndd_file_header_create(&new_header));
//    NDD_VERIFY_RESULT(ndd_file_header_read_from_file(new_header, fp));
//
//    /*  FVD ǡɤ߹  */
//    ndd_fvd_data *new_fvd_data = NULL;
//    NDD_VERIFY_RESULT(ndd_fvd_data_create(&new_fvd_data));
//    NDD_VERIFY_RESULT(ndd_fvd_data_read_from_file(new_fvd_data, new_header, fp));
//
//    /*  ǡǼ  */
//    NDD_IFOK{
//	if(fvd->fvd != NULL){
//	    NDD_VERIFY_RESULT(ndd_fvd_data_destroy(&fvd->fvd, fvd->header));
//	}
//	if(fvd->header != NULL){
//	    NDD_VERIFY_RESULT(ndd_file_header_destroy(&fvd->header));
//	}
//    }
//    NDD_IFOK{
//	fvd->header = new_header;
//	fvd->fvd    = new_fvd_data;
//	new_header   = NULL;
//	new_fvd_data = NULL;
//    }
//
//    if(new_fvd_data != NULL){  ndd_fvd_data_destroy   (&new_fvd_data, new_header);  }
//    if(new_header   != NULL){  ndd_file_header_destroy(&new_header);    }
//
//    NDD_RETURN;
//}
//
//ndd_result
//ndd_fvd_write_to_file(const ndd_fvd *fvd, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd);
//    NDD_VERIFY_POINTER(fp);
//
//    /*  եإå񤭹  */
//    NDD_VERIFY_RESULT(ndd_file_header_write_to_file(fvd->header, fp));
//    
//    /*  FVD ǡ񤭹  */
//    NDD_VERIFY_RESULT(ndd_fvd_data_write_to_file(fvd->fvd, fvd->header, fp));
//
//    NDD_RETURN;
//}
//
//ndd_result
//ndd_fvd_print_to_file(const ndd_fvd *fvd, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd);
//    NDD_VERIFY_POINTER(fp);
//
//    /*  եإå񤭹  */
//    NDD_VERIFY_RESULT(ndd_file_header_print_to_file(fvd->header, fp));
//    
//    /*  FVD ǡ񤭹  */
//    NDD_VERIFY_RESULT(ndd_fvd_data_print_to_file(fvd->fvd, fvd->header, fp));
//
//    NDD_RETURN;
//}

//ndd_result
//ndd_fvd_cut(ndd_fvd *dst_fvd, const ndd_fvd *src_fvd, ndd_uint32 frame_offset, ndd_uint32 frame_count)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(dst_fvd);
//    NDD_VERIFY_POINTER(src_fvd);
//
//    NDD_IFOK{
//	if(dst_fvd == src_fvd){
//	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
//	}else if(dst_fvd->header->frame_count > 0){
//	    NDD_SET_ERROR(NDD_ERROR_DATA_ALREADY_EXISTS);
//	}else if(frame_offset < src_fvd->header->frame_offset){
//	    NDD_SET_ERROR(NDD_ERROR_FRAME_OFFSET_OUT_OF_BOUND);
//	}else if(frame_offset + frame_count > src_fvd->header->frame_offset + src_fvd->header->frame_count){
//	    NDD_SET_ERROR(NDD_ERROR_FRAME_COUNT_OUT_OF_BOUND);
//	}
//    }
//
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_fvd->header, &elements_per_frame));
//    
//    ndd_float8 **new_frame_data = NULL;
//    NDD_VERIFY_RESULT(ndd_calloc(&new_frame_data, sizeof(ndd_float8 *) * frame_count));
//
//    ndd_uint32 i;
//    NDD_FOR(i = 0; i < frame_count; i++){
//	ndd_float8 *f = NULL;
//	NDD_VERIFY_RESULT(ndd_malloc(&f, sizeof(ndd_float8) * elements_per_frame));
//	NDD_IFOK{
//	    memcpy(f, src_fvd->fvd->frame_data[frame_offset - src_fvd->header->frame_offset + i],
//		   sizeof(ndd_float8) * elements_per_frame);
//	}
//
//	NDD_IFOK{
//	    new_frame_data[i] = f;
//	    f = NULL;
//	}
//
//	if(f != NULL){ ndd_free(&f); }
//    }
//
//    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_fvd->header, src_fvd->header));
//    NDD_IFOK{
//	dst_fvd->header->frame_offset = frame_offset;
//	dst_fvd->header->frame_count  = frame_count;
//	dst_fvd->fvd->frame_data      = new_frame_data;
//	new_frame_data = NULL;
//    }
//
//    if(new_frame_data != NULL){
//	for(i = 0; i < frame_count; i++){
//	    if(new_frame_data[i] != NULL){
//		ndd_free(&new_frame_data[i]);
//	    }
//	}
//	ndd_free(&new_frame_data);
//    }
//
//    NDD_RETURN;
//}
//
//ndd_result
//ndd_fvd_concatenate(ndd_fvd *dst_fvd, const ndd_fvd *src_fvd_1, const ndd_fvd *src_fvd_2)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(dst_fvd);
//    NDD_VERIFY_POINTER(src_fvd_1);
//    NDD_VERIFY_POINTER(src_fvd_2);
//
//    NDD_IFOK{
//	if(dst_fvd == src_fvd_2){
//	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
//	}else if(dst_fvd->header->frame_count > 0 && dst_fvd != src_fvd_1){
//	    NDD_SET_ERROR(NDD_ERROR_DATA_ALREADY_EXISTS);
//	}else if(dst_fvd->header->frame_count > 0 &&
//		 src_fvd_1->header->frame_offset + src_fvd_1->header->frame_count != src_fvd_2->header->frame_offset){
//	    NDD_SET_ERROR(NDD_ERROR_FRAME_DATA_NOT_CONTIGUOUS);
//	}
//    }
//
//    NDD_IFOK{
//	if(dst_fvd != src_fvd_1){
//	    NDD_VERIFY_RESULT(ndd_fvd_copy(dst_fvd, src_fvd_1));
//	}
//	if(dst_fvd == src_fvd_1 && dst_fvd->header->frame_count == 0){
//	    return ndd_fvd_copy(dst_fvd, src_fvd_2);
//	}
//    }
//
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_fvd_1->header, &elements_per_frame));
//    
//    ndd_uint32 i;
//    NDD_FOR(i = 0; i < src_fvd_2->header->frame_count; i++){
//	ndd_float8 *f = NULL;
//	NDD_VERIFY_RESULT(ndd_malloc(&f, sizeof(ndd_float8) * elements_per_frame));
//	NDD_IFOK{
//	    memcpy(f, src_fvd_2->fvd->frame_data[i], sizeof(ndd_float8) * elements_per_frame);
//	}
//
//	NDD_VERIFY_RESULT(ndd_realloc(&dst_fvd->fvd->frame_data,
//				      sizeof(ndd_float8 *) * (dst_fvd->header->frame_count + 1)));
//	NDD_IFOK{
//	    dst_fvd->fvd->frame_data[dst_fvd->header->frame_count++] = f;
//	    f = NULL;
//	}
//
//	if(f != NULL){ ndd_free(&f); }
//    }
//
//    NDD_RETURN;
//}

ndd_result
ndd_fvd_cut(ndd_fvd *dst_fvd, ndd_fvd *src_fvd, ndd_uint32 frame_offset, ndd_uint32 frame_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_fvd);
    NDD_VERIFY_POINTER(src_fvd);

    NDD_IFOK{
	if(dst_fvd == src_fvd){
	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
	}else if(dst_fvd->header->frame_count > 0){
	    NDD_SET_ERROR(NDD_ERROR_DATA_ALREADY_EXISTS);
	}else if(frame_offset < src_fvd->header->frame_offset){
	    NDD_SET_ERROR(NDD_ERROR_FRAME_OFFSET_OUT_OF_BOUND);
	}else if(frame_offset + frame_count > src_fvd->header->frame_offset + src_fvd->header->frame_count){
	    NDD_SET_ERROR(NDD_ERROR_FRAME_COUNT_OUT_OF_BOUND);
	}
    }

    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_fvd->header, src_fvd->header));
    NDD_IFOK{
	dst_fvd->header->frame_offset = frame_offset;
	dst_fvd->header->frame_count  = 0;
    }

    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_fvd->header, &elements_per_frame));
    
    ndd_float8 *frame_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&frame_data, sizeof(ndd_float8) * elements_per_frame));

    ndd_uint32 i;
    NDD_FOR(i = 0; i < frame_count; i++){
	NDD_VERIFY_RESULT(ndd_fvd_read_frame_data (src_fvd, frame_offset + i, frame_data, elements_per_frame));
	NDD_VERIFY_RESULT(ndd_fvd_write_frame_data(dst_fvd, frame_offset + i, frame_data, elements_per_frame));
    }

    if(frame_data != NULL){
	ndd_free(&frame_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_append(ndd_fvd *dst_fvd, ndd_fvd *src_fvd)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_fvd);
    NDD_VERIFY_POINTER(src_fvd);

    NDD_IFOK{
	if(dst_fvd == src_fvd){
	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
//	}else if(dst_fvd->header->frame_offset + dst_fvd->header->frame_count != src_fvd->header->frame_offset){
//	    NDD_SET_ERROR(NDD_ERROR_FRAME_DATA_NOT_CONTIGUOUS);
	}
    }

    NDD_IFOK{
	if(dst_fvd->header->frame_count == 0){
	    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_fvd->header, src_fvd->header));
	    NDD_IFOK{
		dst_fvd->header->frame_count = 0;
	    }
	}else{
	    ndd_bool is_same = false;
	    NDD_VERIFY_RESULT(ndd_fvd_compare_format(dst_fvd, src_fvd, &is_same));
	}
    }

    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_fvd->header, &elements_per_frame));
    
    ndd_uint32 frames_to_read = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(src_fvd->header, &frames_to_read));

    ndd_uint32 src_frame_offset = 0;
    ndd_uint32 dst_frame_offset = 0;
    NDD_IFOK{
	src_frame_offset = src_fvd->header->frame_offset;
	dst_frame_offset = dst_fvd->header->frame_offset + dst_fvd->header->frame_count;

    }
    ndd_float8 *f = NULL;
    NDD_VERIFY_RESULT(ndd_malloc(&f, sizeof(ndd_float8) * elements_per_frame));

    NDD_WHILE(frames_to_read > 0){
	NDD_VERIFY_RESULT(ndd_fvd_read_frame_data (src_fvd, src_frame_offset, f, elements_per_frame));
	NDD_VERIFY_RESULT(ndd_fvd_write_frame_data(dst_fvd, dst_frame_offset, f, elements_per_frame));
	NDD_IFOK{
	    src_frame_offset++;
	    dst_frame_offset++;
	    frames_to_read--;
	}
    }

    if(f != NULL){
	ndd_free(&f);
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_compare_format(const ndd_fvd *fvd1, const ndd_fvd *fvd2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(fvd1);
    NDD_VERIFY_POINTER(fvd2);

    NDD_VERIFY_RESULT(ndd_fvd_header_compare_format(fvd1->header, fvd2->header, is_same));

    NDD_RETURN;
}

ndd_result
ndd_fvd_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(header1);
    NDD_VERIFY_POINTER(header2);
    NDD_VERIFY_POINTER(is_same);

    ndd_bool same = false;
    NDD_IFOK{
	if(header1->screen_width  != header2->screen_width ||
	   header1->screen_height != header2->screen_height){
	    NDD_SET_ERROR(NDD_ERROR_SCREEN_SIZE_MISMATCH);
	}else if(header1->region_left != header2->region_left ||
	   header1->region_top  != header2->region_top){
	    NDD_SET_ERROR(NDD_ERROR_REGION_POSITION_MISMATCH);
	}else if(header1->region_width  != header2->region_width ||
	   header1->region_height != header2->region_height){
	    NDD_SET_ERROR(NDD_ERROR_REGION_SIZE_MISMATCH);
	}else if(header1->block_width  != header2->block_width ||
	   header1->block_height != header2->block_height){
	    NDD_SET_ERROR(NDD_ERROR_BLOCK_SIZE_MISMATCH);
	}else if(header1->color_resolution != header2->color_resolution){
	    NDD_SET_ERROR(NDD_ERROR_COLOR_RESOLUTION_MISMATCH);
	}else{
	    same = true;
	}
    }

    NDD_IFOK{
	*is_same = same;
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_data_create(ndd_fvd_data **fvd_data)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(fvd_data);

    /*  ΰ  */
    ndd_fvd_data *new_fvd_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_fvd_data, sizeof(ndd_fvd_data)));

    /*  ѿν  */

    /*  ̤Ǽ  */
    NDD_IFOK{
	*fvd_data = new_fvd_data;
	new_fvd_data = NULL;
    }

    /*  פʥǡդ  */
    if(new_fvd_data != NULL){
	ndd_free(&new_fvd_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_data_destroy(ndd_fvd_data **fvd_data, const ndd_file_header *header)
{
    NDD_START;
    NDD_VERIFY_POINTER(fvd_data);
    NDD_VERIFY_POINTER(header);

    ndd_fvd_data *old_fvd_data;
    NDD_IFOK{
	old_fvd_data = *fvd_data;
    }

    ndd_uint32 frame_count = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));

    ndd_uint32 i, j;
    NDD_FOR(i = 0; i < old_fvd_data->cache_count; i++){
	ndd_fvd_data_cache *cache = old_fvd_data->caches[i];

	NDD_FOR(j = 0; j < cache->frame_count; j++){
	    NDD_VERIFY_RESULT(ndd_free(&cache->frame_data[j]));
	}
	NDD_VERIFY_RESULT(ndd_free(&old_fvd_data->caches[i]));
    }
    NDD_VERIFY_RESULT(ndd_free(&old_fvd_data));
    
    NDD_IFOK{
	*fvd_data = NULL;
    }

    NDD_RETURN;
}

//ndd_result
//ndd_fvd_data_read_from_file(ndd_fvd_data *fvd_data, const ndd_file_header *header, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd_data);
//    NDD_VERIFY_POINTER(header);
//    NDD_VERIFY_POINTER(fp);
//
//    ndd_uint32 frame_count = 0;
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header, &elements_per_frame));
//
//    ndd_float8 **new_frame_data = NULL;
//    NDD_VERIFY_RESULT(ndd_calloc(&new_frame_data, sizeof(ndd_float8 *) * frame_count));
//
//    ndd_uint32 i;
//    NDD_FOR(i = 0; i < frame_count; i++){
//	ndd_float8 *f = NULL;
//	NDD_VERIFY_RESULT(ndd_calloc(&f, sizeof(ndd_float8) * elements_per_frame));
//	NDD_VERIFY((fread(f, sizeof(ndd_float8), elements_per_frame, fp) == elements_per_frame),
//		   NDD_ERROR_FILE_READ_FAILED);
//
//	NDD_IFOK{
//	    new_frame_data[i] = f;
//	    f = NULL;
//	}
//	if(f != NULL){  ndd_free(&f);  }
//    }
//
//    NDD_IFOK{
//	fvd_data->frame_data = new_frame_data;
//	new_frame_data = NULL;
//    }
//
//    if(new_frame_data != NULL){
//	for(i = 0; i < frame_count; i++){ ndd_free(&new_frame_data[i]);  }
//	ndd_free(&new_frame_data);
//    }
//
//    NDD_RETURN;
//}
//
//ndd_result
//ndd_fvd_data_write_to_file(const ndd_fvd_data *fvd_data, const ndd_file_header *header, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd_data);
//    NDD_VERIFY_POINTER(header);
//    NDD_VERIFY_POINTER(fp);
//
//    ndd_uint32 frame_count = 0;
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header, &elements_per_frame));
//
//    ndd_uint32 i;
//    NDD_FOR(i = 0; i < frame_count; i++){
//	NDD_VERIFY((fwrite(fvd_data->frame_data[i], sizeof(ndd_float8), elements_per_frame, fp) == elements_per_frame),
//		   NDD_ERROR_FILE_WRITE_FAILED);
//    }
//
//    NDD_RETURN;
//}
//
//ndd_result
//ndd_fvd_data_print_to_file(const ndd_fvd_data *fvd_data, const ndd_file_header *header, FILE *fp)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(fvd_data);
//    NDD_VERIFY_POINTER(header);
//    NDD_VERIFY_POINTER(fp);
//
//    ndd_uint32 frame_count = 0;
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header, &elements_per_frame));
//
//    ndd_uint32 i, j, k;
//    fputs("FVD data:\n", fp);
//    NDD_FOR(i = 0; i < frame_count; i++){
//	fprintf(fp, "  Frame %lu\n", i);
//	NDD_FOR(j = 0; j < elements_per_frame; j += 8){
//	    fputs("    ", fp);
//	    NDD_FOR(k = 0; k < 8 && j + k < elements_per_frame; k++){
//		fprintf(fp, "%7.4f, ", fvd_data->frame_data[i][j + k]);
//	    }
//	    fputs("\n", fp);
//	}
//	fputs("\n", fp);
//    }
//
//    NDD_RETURN;
//}

//ndd_result
//ndd_fvd_data_copy(ndd_fvd_data *dst_fvd_data, const ndd_fvd_data *src_fvd_data, const ndd_file_header *header)
//{
//    NDD_START;
//    NDD_VERIFY_POINTER(dst_fvd_data);
//    NDD_VERIFY_POINTER(src_fvd_data);
//    NDD_VERIFY_POINTER(header);
//
//    ndd_uint32 frame_count = 0;
//    ndd_uint32 elements_per_frame = 0;
//    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count       (header, &frame_count));
//    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header, &elements_per_frame));
//
//    ndd_float8 **new_frame_data = NULL;
//    NDD_VERIFY_RESULT(ndd_calloc(&new_frame_data, sizeof(ndd_float8 *) * frame_count));
//
//    ndd_uint32 i;
//    NDD_FOR(i = 0; i < frame_count; i++){
//	ndd_float8 *f = NULL;
//	NDD_VERIFY_RESULT(ndd_calloc(&f, sizeof(ndd_float8) * elements_per_frame));
//	NDD_IFOK{
//	    memcpy(f, src_fvd_data->frame_data[i], sizeof(ndd_float8) * elements_per_frame);
//	}
//
//	NDD_IFOK{
//	    new_frame_data[i] = f;
//	    f = NULL;
//	}
//	if(f != NULL){  ndd_free(&f);  }
//    }
//
//    NDD_IFOK{
//	dst_fvd_data->frame_data = new_frame_data;
//	new_frame_data = NULL;
//    }
//
//    if(new_frame_data != NULL){
//	for(i = 0; i < frame_count; i++){ ndd_free(&new_frame_data[i]);  }
//	ndd_free(&new_frame_data);
//    }
//
//    NDD_RETURN;
//}

ndd_result
ndd_fvd_assign_file(ndd_fvd *fvd, const char *filename, ndd_file_mode file_mode)
{
    NDD_START;

    FILE *fp = NULL;
    NDD_VERIFY((fp = fopen64(filename, (file_mode == ndd_file_mode_write) ? "w" : "r")) != NULL, NDD_ERROR_FILE_OPEN_FAILED);

    NDD_IFOK{
	switch(file_mode){
	case ndd_file_mode_read:
	    NDD_VERIFY_RESULT(ndd_file_header_read_from_file(fvd->header, fp));
	    break;

	case ndd_file_mode_write:
	    NDD_VERIFY_RESULT(ndd_file_header_write_to_file(fvd->header, fp));
	    break;

	default:
	    NDD_SET_ERROR(NDD_ERROR_INVALID_FILE_MODE);
	}
    }

    off_t frame_data_offset = 0;
    NDD_IFOK{
	frame_data_offset = ftello64(fp);
    }

    NDD_IFOK{
	fvd->file              = fp;
	fvd->file_mode         = file_mode;
	fvd->frame_data_offset = frame_data_offset;
	fvd->fvd->cache_count  = 0;
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_read_frame_data(ndd_fvd *fvd, ndd_uint32 frame_index,
			ndd_float8 *frame_data, ndd_uint32 frame_data_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(fvd);
    NDD_VERIFY_POINTER(frame_data);

    /*  ɤ߹ߤ˻ѤǤ FVD ǡ  */
    NDD_IFOK{
	if(fvd->file == NULL){
	    NDD_SET_ERROR(NDD_ERROR_FILE_NOT_ASSIGNED);
	}else if(fvd->file_mode != ndd_file_mode_read){
	    NDD_SET_ERROR(NDD_ERROR_FILE_NOT_OPENED_TO_READ);
	}
    }

    /*  frame_index оݤFVD ǡ˴ޤޤ뤫  */
    NDD_IFOK{
	if(frame_index < fvd->header->frame_offset || fvd->header->frame_offset + fvd->header->frame_count <= frame_index){
	    NDD_SET_ERROR(NDD_ERROR_FRAME_INDEX_OUT_OF_BOUND);
	}
    }

    /*  ե졼ॵ  */
    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(fvd->header, &elements_per_frame));

//    fprintf(stdout, "[%p] Read %lu\n", fvd, frame_index);

    /*  å򸡺  */
    ndd_fvd_data_cache *active_cache = NULL;
    ndd_uint32 cache_index;
    NDD_FOR(cache_index = 0; cache_index < fvd->fvd->cache_count; cache_index++){
	ndd_fvd_data_cache *cache = fvd->fvd->caches[cache_index];

	/*  å˥ҥåȡ  */
	if(cache->frame_offset <= frame_index && frame_index < cache->frame_offset + cache->frame_count){

	    /*  å˥ҥåȤΤǡ֤ؤ  */
	    memmove(fvd->fvd->caches + 1, fvd->fvd->caches, sizeof(ndd_fvd_data_cache *) * cache_index);
	    fvd->fvd->caches[0] = cache;

//	    fprintf(stdout, "[%p] Cache hit %lu\n", fvd, cache_index);
	    active_cache = cache;
	    break;
	}
    }

    NDD_IFOK{
	/*  åҥåȤʤä  */
	if(active_cache == NULL){

	    ndd_fvd_data_cache *new_cache = NULL;
	    /*  åκãƤ롩  */
	    if(fvd->fvd->cache_count >= NDD_FVD_CACHE_COUNT){
		/*  åκãƤ硢ΥåǺѤ  */
		new_cache = fvd->fvd->caches[fvd->fvd->cache_count - 1];
		fvd->fvd->cache_count--;
	    }else{
		/*  åκãƤʤϡ˺  */
		NDD_VERIFY_RESULT(ndd_calloc(&new_cache, sizeof(ndd_fvd_data_cache)));
	    }

	    /*  ɤɤ߹फ  */
	    ndd_uint32 read_head = frame_index - fvd->header->frame_offset;
	    ndd_uint32 read_tail = read_head + NDD_FVD_CACHE_SIZE;
	    if(read_tail > fvd->header->frame_count){
		read_tail = fvd->header->frame_count;
	    }
	    if(read_head + NDD_FVD_CACHE_SIZE > read_tail){
		read_head = (read_tail > NDD_FVD_CACHE_SIZE ? read_tail - NDD_FVD_CACHE_SIZE : 0);
	    }
	    ndd_uint32 read_offset = read_head;
	    ndd_uint32 read_count  = read_tail - read_head;

	    /*  åΰγ  */
	    ndd_uint32 i;
	    NDD_FOR(i = new_cache->frame_count; i < read_count; i++){
		NDD_VERIFY_RESULT(ndd_malloc(&new_cache->frame_data[i], sizeof(ndd_float8) * elements_per_frame));
	    }
	    NDD_FOR(i = read_count; i < new_cache->frame_count; i++){
		NDD_VERIFY_RESULT(ndd_free(&new_cache->frame_data[i]));
	    }
	    
	    /*  ɤ߹߳ϰ֤إեݥ󥿤ư  */
	    NDD_VERIFY(fseeko64(fvd->file,
			      (off_t)fvd->frame_data_offset + (off_t)sizeof(ndd_float8) * elements_per_frame * read_offset, SEEK_SET) == 0,
		       NDD_ERROR_FILE_SEEK_FAILED);
	    
	    /*  ɤ߹  */
	    NDD_FOR(i = 0; i < read_count; i++){
		NDD_VERIFY(fread(new_cache->frame_data[i], sizeof(ndd_float8), elements_per_frame, fvd->file) == elements_per_frame,
			   NDD_ERROR_FILE_READ_FAILED);
	    }

	    /*  åǼ  */
	    NDD_IFOK{
		new_cache->frame_offset = read_offset + fvd->header->frame_offset;
		new_cache->frame_count  = read_count;
	    }

	    /*  åꥹȤ¸  */
	    NDD_IFOK{
		memmove(fvd->fvd->caches + 1, fvd->fvd->caches,
			sizeof(ndd_fvd_data_cache *) * fvd->fvd->cache_count);
		fvd->fvd->caches[0] = new_cache;
		fvd->fvd->cache_count++;

		active_cache = new_cache;
		new_cache = NULL;
	    }

	    /*  դ  */
	    if(new_cache != NULL){
		for(i = 0; i < new_cache->frame_count; i++){
		    ndd_free(&new_cache->frame_data[i]);
		}
		ndd_free(&new_cache);
	    }
	}
    }

    /*  ǡ򥳥ԡ  */
    NDD_IFOK{
	memcpy(frame_data, active_cache->frame_data[frame_index - active_cache->frame_offset],
	       sizeof(ndd_float8) * elements_per_frame);
    }

    NDD_RETURN;
}

ndd_result
ndd_fvd_write_frame_data(ndd_fvd *fvd, ndd_uint32 frame_index,
			 const ndd_float8 *frame_data, ndd_uint32 frame_data_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(fvd);
    NDD_VERIFY_POINTER(frame_data);

    /*  񤭹ߤ˻ѤǤ FVD ǡ  */
    NDD_IFOK{
	if(fvd->file == NULL){
	    NDD_SET_ERROR(NDD_ERROR_FILE_NOT_ASSIGNED);
	}else if(fvd->file_mode != ndd_file_mode_write){
	    NDD_SET_ERROR(NDD_ERROR_FILE_NOT_OPENED_TO_WRITE);
	}
    }

    /*  frame_index оݤFVD ǡ˴ޤޤ뤫뤤ˤ뤫  */
    NDD_IFOK{
	if(frame_index < fvd->header->frame_offset || fvd->header->frame_offset + fvd->header->frame_count < frame_index){
	    NDD_SET_ERROR(NDD_ERROR_FRAME_INDEX_OUT_OF_BOUND);
	}
    }

    /*  ե졼ॵ  */
    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(fvd->header, &elements_per_frame));

    /*  ǡ񤭹߳ϰ֤إեݥ󥿤ư  */
    NDD_VERIFY(fseeko64(fvd->file, 
		      (off_t)fvd->frame_data_offset + (off_t)sizeof(ndd_float8) * elements_per_frame * (frame_index - fvd->header->frame_offset), SEEK_SET) == 0,
	       NDD_ERROR_FILE_SEEK_FAILED);
	    
    /*  ǡ񤭹  */
    NDD_VERIFY(fwrite(frame_data, sizeof(ndd_float8), elements_per_frame, fvd->file) == elements_per_frame,
	       NDD_ERROR_FILE_READ_FAILED);

    /*  ե졼  */
    NDD_IFOK{
	if(frame_index == fvd->header->frame_offset + fvd->header->frame_count){
	    fvd->header->frame_count++;
	}
    }	

    /*  إå񤭹߳ϰ֤إեݥ󥿤ư  */
    NDD_VERIFY(fseeko64(fvd->file, 0, SEEK_SET) == 0, NDD_ERROR_FILE_SEEK_FAILED);

    /*  إå񤭹  */
    NDD_VERIFY_RESULT(ndd_file_header_write_to_file(fvd->header, fvd->file));

    NDD_RETURN;
}

/*  end of file  */
