/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/memory.h>
#include <ndd/error.h>

#include <stdlib.h>

/*    */

ndd_result
ndd_malloc_impl(void **p, ndd_uint32 size)
{
    NDD_START;
    NDD_VERIFY_POINTER(p);
    
    void *new_p = NULL;
    NDD_IFOK{
	new_p = malloc(size);
	if(new_p == 0){
	    NDD_SET_ERROR(NDD_ERROR_ALLOCATION_FAILED);
	}else{
	    *p = new_p;
	}
    }

    NDD_RETURN;
}

ndd_result
ndd_calloc_impl(void **p, ndd_uint32 size)
{
    NDD_START;
    NDD_VERIFY_POINTER(p);
    
    void *new_p = NULL;
    NDD_IFOK{
	new_p = calloc(size, 1);
	if(new_p == 0){
	    NDD_SET_ERROR(NDD_ERROR_ALLOCATION_FAILED);
	}else{
	    *p = new_p;
	}
    }

    NDD_RETURN;
}

ndd_result
ndd_realloc_impl(void **p, ndd_uint32 size)
{
    NDD_START;
    NDD_VERIFY_POINTER(p);
    
    void *old_p = NULL;
    void *new_p = NULL;
    NDD_IFOK{
	old_p = *p;
	new_p = realloc(old_p, size);
	if(new_p == 0){
	    NDD_SET_ERROR(NDD_ERROR_ALLOCATION_FAILED);
	}else{
	    *p = new_p;
	}
    }

    NDD_RETURN;
}

ndd_result
ndd_free_impl(void **p)
{
    NDD_START;
    NDD_VERIFY_POINTER(p);
    
    void *old_p = NULL;
    NDD_IFOK{
	old_p = *p;
	if(old_p != NULL){
	    free(old_p);
	}
    }

    NDD_RETURN;
}

/*  end of file  */
