/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/mkffd.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/fvd.h>
#include <ndd/ffs.h>
#include <ndd/ffd.h>
#include <ndd/file.h>

#include <math.h>

ndd_result
ndd_create_ffd_from_fvd(ndd_ffd *dst_ffd, ndd_fvd *src_fvd, const ndd_ffs *src_ffs, ndd_callback callback, void *user_data)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_ffd);
    NDD_VERIFY_POINTER(src_fvd);
    NDD_VERIFY_POINTER(src_ffs);

    ndd_bool is_same = false;
    NDD_VERIFY_RESULT(ndd_fvd_header_compare_format(src_fvd->header, src_ffs->header, &is_same));

    /*  Ƽѥ᡼ͤΥåȼ  */
    ndd_uint32 elements_per_frame = 0;
    NDD_IFOK{
	ndd_uint32 fvd_elements_per_frame = 0;
	ndd_uint32 ffs_elements_per_frame = 0;
	NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_fvd->header, &fvd_elements_per_frame));
	NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_ffs->header, &ffs_elements_per_frame));
	if(fvd_elements_per_frame != ffs_elements_per_frame){
	    NDD_SET_ERROR(NDD_ERROR_INVALID_PARAMETER);
	}else{
	    elements_per_frame = fvd_elements_per_frame;
	}
    }

    ndd_uint32 frame_dimension = 0;
    NDD_IFOK{
	frame_dimension = src_ffs->header->frame_dimension;
    }

    ndd_uint32 fvd_frame_count = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(src_fvd->header, &fvd_frame_count));

    ndd_float8 *frame_buffer = NULL;
    NDD_VERIFY_RESULT(ndd_malloc(&frame_buffer, sizeof(ndd_float8) * elements_per_frame));

    /*  ͿΨͤ˽äơͭ٥ȥ  */
    ndd_float8 **new_vector_data  = NULL;
    ndd_uint32   new_vector_count = 0;
    ndd_uint32 i;
    NDD_FOR(i = 0; i < fvd_frame_count; i++){
	NDD_IFOK{
	    if(callback != NULL){
		NDD_VERIFY_RESULT(callback("mkffd", "calculate vector data", i, fvd_frame_count, user_data));
	    }
	}

	NDD_VERIFY_RESULT(ndd_fvd_read_frame_data(src_fvd, src_fvd->header->frame_offset + i,
						  frame_buffer, elements_per_frame));

	ndd_float8 *new_vector = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&new_vector, sizeof(ndd_float8) * frame_dimension));

	ndd_uint32 j, k;
	NDD_FOR(j = 0; j < frame_dimension; j++){
	    const ndd_float8 *v2 = src_ffs->ffs->selector_vector[j];

	    ndd_float8 value = 0;
	    NDD_FOR(k = 0; k < elements_per_frame; k++){
		value += frame_buffer[k] * v2[k];
	    }
	    new_vector[j] = value;
	}

	NDD_VERIFY_RESULT(ndd_realloc(&new_vector_data, sizeof(ndd_float8 *) * (new_vector_count + 1)));
	NDD_IFOK{
	    new_vector_data[new_vector_count++] = new_vector;
	    new_vector = NULL;
	}

	if(new_vector != NULL){  ndd_free(&new_vector);  }
    }
    NDD_IFOK{
	if(callback != NULL){
	    NDD_VERIFY_RESULT(callback("mkffd", NULL, 0, 0, user_data));
	}
    }

    if(frame_buffer != NULL){ ndd_free(&frame_buffer); }

    /*  ͤǼ  */
    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_ffd->header, src_fvd->header));
    NDD_IFOK{
	dst_ffd->header->marker             = NDD_FILE_MARKER_FFD;
	dst_ffd->header->version            = NDD_FILE_VERSION_FFD;
	dst_ffd->header->frame_sample_count = src_ffs->header->frame_sample_count;
	dst_ffd->header->frame_dimension    = src_ffs->header->frame_dimension;
	dst_ffd->header->frame_proportion   = src_ffs->header->frame_proportion;
	dst_ffd->ffd->frame_vector = new_vector_data;
	new_vector_data = NULL;
    }
    NDD_VERIFY_RESULT(ndd_ffs_data_copy(dst_ffd->ffs, src_ffs->ffs, dst_ffd->header));

    /*  դ  */
    if(new_vector_data != NULL){
	for(i = 0; i < new_vector_count; i++){
	    ndd_free(&new_vector_data[i]);
	}
	ndd_free(&new_vector_data);
    }

    NDD_RETURN;
}

/*  end of file  */
