/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/mksfd.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/ffs.h>
#include <ndd/ffd.h>
#include <ndd/sfs.h>
#include <ndd/sfd.h>
#include <ndd/file.h>

#include <math.h>

ndd_result
ndd_create_sfd_from_ffd(ndd_sfd *dst_sfd, const ndd_ffd *src_ffd, const ndd_sfs *src_sfs, ndd_callback callback, void *user_data)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfd);
    NDD_VERIFY_POINTER(src_ffd);
    NDD_VERIFY_POINTER(src_sfs);

    ndd_bool is_same = false;
    NDD_VERIFY_RESULT(ndd_ffd_header_compare_format(src_ffd->header, src_sfs->header, &is_same));
    NDD_VERIFY_RESULT(ndd_ffs_data_compare(src_ffd->ffs, src_ffd->header,
					   src_sfs->ffs, src_sfs->header, &is_same));

    /*  Ƽѥ᡼ͤΥåȼ  */
    ndd_uint32 frame_dimension = 0;
    ndd_uint32 segment_dimension = 0;
    ndd_uint32 segment_size      = 0;
    NDD_IFOK{
	segment_size      = src_sfs->header->segment_size;
	segment_dimension = src_sfs->header->segment_dimension;
	frame_dimension   = src_sfs->header->frame_dimension;
    }

    ndd_uint32 ffd_frame_count = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(src_ffd->header, &ffd_frame_count));

    /*  ͿΨͤ˽äơͭ٥ȥ  */
    ndd_float8 **new_vector_data  = NULL;
    ndd_uint32   new_vector_count = 0;
    ndd_uint32 i;
    NDD_FOR(i = 0; i + segment_size < ffd_frame_count; i++){

	NDD_IFOK{
	    if(callback != NULL){
		NDD_VERIFY_RESULT(callback("mksfd", "calculate vector data", i, ffd_frame_count, user_data));
	    }
	}

	ndd_float8 *new_vector = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&new_vector, sizeof(ndd_float8) * segment_dimension));

	ndd_uint32 j, k, l;
	NDD_FOR(j = 0; j < segment_dimension; j++){

	    ndd_float8 value = 0;
	    NDD_FOR(k = 0; k < segment_size; k++){
		const ndd_float8 *v1 = src_sfs->sfs->selector_vector[j] + frame_dimension * k;
		const ndd_float8 *v2 = src_ffd->ffd->frame_vector[i + k];

		NDD_FOR(l = 0; l < frame_dimension; l++){
		    value += v1[k] * v2[k];
		}
	    }
	    new_vector[j] = value;
	}

	NDD_VERIFY_RESULT(ndd_realloc(&new_vector_data, sizeof(ndd_float8 *) * (new_vector_count + 1)));
	NDD_IFOK{
	    new_vector_data[new_vector_count++] = new_vector;
	    new_vector = NULL;
	}

	if(new_vector != NULL){  ndd_free(&new_vector);  }
    }
    NDD_IFOK{
	if(callback != NULL){
	    NDD_VERIFY_RESULT(callback("mksfd", NULL, 0, 0, user_data));
	}
    }

    /*  ͤǼ  */
    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_sfd->header, src_ffd->header));
    NDD_IFOK{
	dst_sfd->header->marker               = NDD_FILE_MARKER_SFD;
	dst_sfd->header->version              = NDD_FILE_VERSION_SFD;
	dst_sfd->header->segment_sample_count = src_sfs->header->segment_sample_count;
	dst_sfd->header->segment_dimension    = src_sfs->header->segment_dimension;
	dst_sfd->header->segment_proportion   = src_sfs->header->segment_proportion;
	dst_sfd->header->segment_size         = src_sfs->header->segment_size;
	dst_sfd->header->total_segment_count  = src_ffd->header->total_frame_count;
	dst_sfd->header->segment_offset       = src_ffd->header->frame_offset;
	dst_sfd->header->segment_count        = src_ffd->header->frame_count - src_sfs->header->segment_size;
	dst_sfd->header->segments_per_second  = src_ffd->header->frames_per_second;
	dst_sfd->sfd->segment_vector = new_vector_data;
	new_vector_data = NULL;
    }
    NDD_VERIFY_RESULT(ndd_ffs_data_copy(dst_sfd->ffs, src_sfs->ffs, dst_sfd->header));
    NDD_VERIFY_RESULT(ndd_sfs_data_copy(dst_sfd->sfs, src_sfs->sfs, dst_sfd->header));

    /*  դ  */
    if(new_vector_data != NULL){
	for(i = 0; i < new_vector_count; i++){
	    ndd_free(&new_vector_data[i]);
	}
	ndd_free(&new_vector_data);
    }

    NDD_RETURN;
}

/*  end of file  */
