/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/sfd.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/file.h>
#include <ndd/ffs.h>
#include <ndd/sfs.h>

#include <string.h>

ndd_result
ndd_sfd_create(ndd_sfd **sfd)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(sfd);

    /*  ΰ  */
    ndd_sfd *new_sfd = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_sfd, sizeof(ndd_sfd)));

    /*  ѿν  */
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_sfd->header));
    NDD_VERIFY_RESULT(ndd_ffs_data_create   (&new_sfd->ffs));
    NDD_VERIFY_RESULT(ndd_sfs_data_create   (&new_sfd->sfs));
    NDD_VERIFY_RESULT(ndd_sfd_data_create   (&new_sfd->sfd));

    /*  ̤Ǽ  */
    NDD_IFOK{
	*sfd = new_sfd;
	new_sfd = NULL;
    }

    /*  פʥǡդ  */
    if(new_sfd != NULL){
	ndd_sfd_destroy(&new_sfd);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_destroy(ndd_sfd **sfd)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd);

    ndd_sfd *old_sfd;
    NDD_IFOK{
	old_sfd = *sfd;
    }

    NDD_VERIFY_RESULT(ndd_sfd_data_destroy   (&old_sfd->sfd, old_sfd->header));
    NDD_VERIFY_RESULT(ndd_sfs_data_destroy   (&old_sfd->sfs, old_sfd->header));
    NDD_VERIFY_RESULT(ndd_ffs_data_destroy   (&old_sfd->ffs, old_sfd->header));
    NDD_VERIFY_RESULT(ndd_file_header_destroy(&old_sfd->header));
    NDD_VERIFY_RESULT(ndd_free(&old_sfd));
    
    NDD_IFOK{
	*sfd = NULL;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_copy(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfd);
    NDD_VERIFY_POINTER(src_sfd);

    /*  إå򥳥ԡ  */
    ndd_file_header *new_header = NULL;
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_header));
    NDD_VERIFY_RESULT(ndd_file_header_copy(new_header, src_sfd->header));

    /*  FFS ǡ򥳥ԡ  */
    ndd_ffs_data *new_ffs_data = NULL;
    NDD_VERIFY_RESULT(ndd_ffs_data_create(&new_ffs_data));
    NDD_VERIFY_RESULT(ndd_ffs_data_copy(new_ffs_data, src_sfd->ffs, src_sfd->header));

    /*  SFS ǡ򥳥ԡ  */
    ndd_sfs_data *new_sfs_data = NULL;
    NDD_VERIFY_RESULT(ndd_sfs_data_create(&new_sfs_data));
    NDD_VERIFY_RESULT(ndd_sfs_data_copy(new_sfs_data, src_sfd->sfs, src_sfd->header));

    /*  SFD ǡ򥳥ԡ  */
    ndd_sfd_data *new_sfd_data = NULL;
    NDD_VERIFY_RESULT(ndd_sfd_data_create(&new_sfd_data));
    NDD_VERIFY_RESULT(ndd_sfd_data_copy(new_sfd_data, src_sfd->sfd, src_sfd->header));

    /*  ǡǼ  */
    NDD_IFOK{
	if(dst_sfd->sfd != NULL){
	    NDD_VERIFY_RESULT(ndd_sfd_data_destroy(&dst_sfd->sfd, dst_sfd->header));
	}
	if(dst_sfd->sfs != NULL){
	    NDD_VERIFY_RESULT(ndd_sfs_data_destroy(&dst_sfd->sfs, dst_sfd->header));
	}
	if(dst_sfd->ffs != NULL){
	    NDD_VERIFY_RESULT(ndd_ffs_data_destroy(&dst_sfd->ffs, dst_sfd->header));
	}
	if(dst_sfd->header != NULL){
	    NDD_VERIFY_RESULT(ndd_file_header_destroy(&dst_sfd->header));
	}
    }
    NDD_IFOK{
	dst_sfd->header = new_header;
	dst_sfd->sfd    = new_sfd_data;
	dst_sfd->ffs    = new_ffs_data;
	dst_sfd->sfs    = new_sfs_data;
	new_header   = NULL;
	new_sfd_data = NULL;
	new_ffs_data = NULL;
	new_sfs_data = NULL;
    }

    if(new_sfd_data != NULL){  ndd_sfd_data_destroy   (&new_sfd_data, new_header);  }
    if(new_sfs_data != NULL){  ndd_sfs_data_destroy   (&new_sfs_data, new_header);  }
    if(new_ffs_data != NULL){  ndd_ffs_data_destroy   (&new_ffs_data, new_header);  }
    if(new_header   != NULL){  ndd_file_header_destroy(&new_header);    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_cut(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd, ndd_uint32 segment_offset, ndd_uint32 segment_count)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfd);
    NDD_VERIFY_POINTER(src_sfd);

    NDD_IFOK{
	if(dst_sfd == src_sfd){
	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
	}else if(dst_sfd->header->segment_count > 0){
	    NDD_SET_ERROR(NDD_ERROR_DATA_ALREADY_EXISTS);
	}else if(segment_offset < src_sfd->header->segment_offset){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_OFFSET_OUT_OF_BOUND);
	}else if(segment_offset + segment_count > src_sfd->header->segment_offset + src_sfd->header->segment_count){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_COUNT_OUT_OF_BOUND);
	}
    }

    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_sfd->header, &elements_per_frame));
    
    ndd_float8 **new_segment_vector = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_segment_vector, sizeof(ndd_float8 *) * segment_count));

    ndd_uint32 i;
    NDD_FOR(i = 0; i < segment_count; i++){
	ndd_float8 *f = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&f, sizeof(ndd_float8) * elements_per_frame));
	NDD_IFOK{
	    memcpy(f, src_sfd->sfd->segment_vector[segment_offset - src_sfd->header->segment_offset + i],
		   sizeof(ndd_float8) * elements_per_frame);
	}

	NDD_IFOK{
	    new_segment_vector[i] = f;
	    f = NULL;
	}

	if(f != NULL){ ndd_free(&f); }
    }

    NDD_VERIFY_RESULT(ndd_file_header_copy(dst_sfd->header, src_sfd->header));
    NDD_VERIFY_RESULT(ndd_ffs_data_copy   (dst_sfd->ffs,    src_sfd->ffs, src_sfd->header));
    NDD_VERIFY_RESULT(ndd_sfs_data_copy   (dst_sfd->sfs,    src_sfd->sfs, src_sfd->header));
    NDD_IFOK{
	dst_sfd->header->segment_offset = segment_offset;
	dst_sfd->header->segment_count  = segment_count;
	dst_sfd->sfd->segment_vector    = new_segment_vector;
	new_segment_vector = NULL;
    }

    if(new_segment_vector != NULL){
	for(i = 0; i < segment_count; i++){
	    if(new_segment_vector[i] != NULL){
		ndd_free(&new_segment_vector[i]);
	    }
	}
	ndd_free(&new_segment_vector);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_concatenate(ndd_sfd *dst_sfd, const ndd_sfd *src_sfd_1, const ndd_sfd *src_sfd_2)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfd);
    NDD_VERIFY_POINTER(src_sfd_1);
    NDD_VERIFY_POINTER(src_sfd_2);

    NDD_IFOK{
	if(dst_sfd == src_sfd_2){
	    NDD_SET_ERROR(NDD_ERROR_SAME_DATA);
	}else if(dst_sfd->header->segment_count > 0 && dst_sfd != src_sfd_1){
	    NDD_SET_ERROR(NDD_ERROR_DATA_ALREADY_EXISTS);
	}else if(dst_sfd->header->segment_count > 0 &&
		 src_sfd_1->header->segment_offset + src_sfd_1->header->segment_count != src_sfd_2->header->segment_offset){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_DATA_NOT_CONTIGUOUS);
	}
    }

    NDD_IFOK{
	if(dst_sfd != src_sfd_1){
	    NDD_VERIFY_RESULT(ndd_sfd_copy(dst_sfd, src_sfd_1));
	}
	if(dst_sfd == src_sfd_1 && dst_sfd->header->segment_count == 0){
	    return ndd_sfd_copy(dst_sfd, src_sfd_2);
	}
    }

    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(src_sfd_1->header, &elements_per_frame));
    
    ndd_uint32 i;
    NDD_FOR(i = 0; i < src_sfd_2->header->segment_count; i++){
	ndd_float8 *f = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&f, sizeof(ndd_float8) * elements_per_frame));
	NDD_IFOK{
	    memcpy(f, src_sfd_2->sfd->segment_vector[i], sizeof(ndd_float8) * elements_per_frame);
	}

	NDD_VERIFY_RESULT(ndd_realloc(&dst_sfd->sfd->segment_vector,
				      sizeof(ndd_float8 *) * (dst_sfd->header->segment_count + 1)));
	NDD_IFOK{
	    dst_sfd->sfd->segment_vector[dst_sfd->header->segment_count++] = f;
	    f = NULL;
	}

	if(f != NULL){ ndd_free(&f); }
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_read_from_file(ndd_sfd *sfd, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd);
    NDD_VERIFY_POINTER(fp);

    /*  إåɤ߹  */
    ndd_file_header *new_header = NULL;
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_header));
    NDD_VERIFY_RESULT(ndd_file_header_read_from_file(new_header, fp));

    /*  FFS ǡɤ߹  */
    ndd_ffs_data *new_ffs_data = NULL;
    NDD_VERIFY_RESULT(ndd_ffs_data_create(&new_ffs_data));
    NDD_VERIFY_RESULT(ndd_ffs_data_read_from_file(new_ffs_data, new_header, fp));

    /*  sfs ǡɤ߹  */
    ndd_sfs_data *new_sfs_data = NULL;
    NDD_VERIFY_RESULT(ndd_sfs_data_create(&new_sfs_data));
    NDD_VERIFY_RESULT(ndd_sfs_data_read_from_file(new_sfs_data, new_header, fp));
    
    /*  sfd ǡɤ߹  */
    ndd_sfd_data *new_sfd_data = NULL;
    NDD_VERIFY_RESULT(ndd_sfd_data_create(&new_sfd_data));
    NDD_VERIFY_RESULT(ndd_sfd_data_read_from_file(new_sfd_data, new_header, fp));

    /*  ǡǼ  */
    NDD_IFOK{
	if(sfd->ffs != NULL){
	    NDD_VERIFY_RESULT(ndd_ffs_data_destroy(&sfd->ffs, sfd->header));
	}
	if(sfd->sfs != NULL){
	    NDD_VERIFY_RESULT(ndd_sfs_data_destroy(&sfd->sfs, sfd->header));
	}
	if(sfd->sfd != NULL){
	    NDD_VERIFY_RESULT(ndd_sfd_data_destroy(&sfd->sfd, sfd->header));
	}
	if(sfd->header != NULL){
	    NDD_VERIFY_RESULT(ndd_file_header_destroy(&sfd->header));
	}
    }
    NDD_IFOK{
	sfd->header = new_header;
	sfd->ffs    = new_ffs_data;
	sfd->sfs    = new_sfs_data;
	sfd->sfd    = new_sfd_data;
	new_header   = NULL;
	new_ffs_data = NULL;
	new_sfs_data = NULL;
	new_sfd_data = NULL;
    }

    if(new_ffs_data != NULL){  ndd_ffs_data_destroy   (&new_ffs_data, new_header);  }
    if(new_sfs_data != NULL){  ndd_sfs_data_destroy   (&new_sfs_data, new_header);  }
    if(new_sfd_data != NULL){  ndd_sfd_data_destroy   (&new_sfd_data, new_header);  }
    if(new_header   != NULL){  ndd_file_header_destroy(&new_header);    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_write_to_file(const ndd_sfd *sfd, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd);
    NDD_VERIFY_POINTER(fp);

    /*  եإå񤭹  */
    NDD_VERIFY_RESULT(ndd_file_header_write_to_file(sfd->header, fp));

    /*  FFS ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_ffs_data_write_to_file(sfd->ffs, sfd->header, fp));

    /*  sfd ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfs_data_write_to_file(sfd->sfs, sfd->header, fp));
	
    /*  sfd ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfd_data_write_to_file(sfd->sfd, sfd->header, fp));

    NDD_RETURN;
}

ndd_result
ndd_sfd_print_to_file(const ndd_sfd *sfd, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd);
    NDD_VERIFY_POINTER(fp);

    /*  եإå񤭹  */
    NDD_VERIFY_RESULT(ndd_file_header_print_to_file(sfd->header, fp));

    /*  FFS ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_ffs_data_print_to_file(sfd->ffs, sfd->header, fp));

    /*  sfd ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfs_data_print_to_file(sfd->sfs, sfd->header, fp));
	
    /*  sfd ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfd_data_print_to_file(sfd->sfd, sfd->header, fp));

    NDD_RETURN;
}

ndd_result
ndd_sfd_compare_format(const ndd_sfd *sfd1, const ndd_sfd *sfd2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd1);
    NDD_VERIFY_POINTER(sfd2);
    NDD_VERIFY_POINTER(is_same);
    
    NDD_VERIFY_RESULT(ndd_sfd_header_compare_format(sfd1->header, sfd2->header, is_same));
    NDD_VERIFY_RESULT(ndd_ffs_data_compare(sfd1->ffs, sfd1->header,
					   sfd2->ffs, sfd2->header, is_same));
    NDD_VERIFY_RESULT(ndd_sfs_data_compare(sfd1->sfs, sfd1->header,
					   sfd2->sfs, sfd2->header, is_same));
    
    NDD_RETURN;
}

ndd_result
ndd_sfd_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(header1);
    NDD_VERIFY_POINTER(header2);
    NDD_VERIFY_POINTER(is_same);

    ndd_bool same = false;
    NDD_VERIFY_RESULT(ndd_ffs_header_compare_format(header1, header2, &same));

    NDD_IFOK{
	*is_same = same;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_create(ndd_sfd_data **sfd_data)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(sfd_data);

    /*  ΰ  */
    ndd_sfd_data *new_sfd_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_sfd_data, sizeof(ndd_sfd_data)));

    /*  ѿν  */

    /*  ̤Ǽ  */
    NDD_IFOK{
	*sfd_data = new_sfd_data;
	new_sfd_data = NULL;
    }

    /*  פʥǡդ  */
    if(new_sfd_data != NULL){
	ndd_free(&new_sfd_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_destroy(ndd_sfd_data **sfd_data, const ndd_file_header *header)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfd_data);
    NDD_VERIFY_POINTER(header);

    ndd_sfd_data *old_sfd_data;
    NDD_IFOK{
	old_sfd_data = *sfd_data;
    }

    //ndd_uint32 frame_count = 0;
    //NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));

    ndd_uint32 i;
    NDD_FOR(i = 0; i < header->segment_dimension; i++){
	NDD_VERIFY_RESULT(ndd_free(&old_sfd_data->segment_vector[i]));
    }
    NDD_VERIFY_RESULT(ndd_free(&old_sfd_data->segment_vector));
    NDD_VERIFY_RESULT(ndd_free(&old_sfd_data));
    
    NDD_IFOK{
	*sfd_data = NULL;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_read_from_file(ndd_sfd_data *sfd_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

    /*  ե졼礭ʤɤ  */
    ndd_uint32 segment_count =0;
    NDD_VERIFY_RESULT(ndd_file_header_get_segment_count(header, &segment_count));
    	
     /*  ɤ߹ sfd ǡǼΰ  */
    ndd_float8 **new_vector_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_vector_data, sizeof(ndd_float8 ) * segment_count));

    /*  ƥե졼Υǡɤ߹  */
    ndd_uint32 i;
    NDD_FOR(i = 0 ; i < segment_count; i++){

	ndd_float8 *vector_data = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&vector_data, sizeof(ndd_float8) * header->segment_dimension));
	NDD_VERIFY(fread(vector_data, sizeof(ndd_float8), header->segment_dimension, fp) == header->segment_dimension,
		   NDD_ERROR_FILE_READ_FAILED);
	
	NDD_IFOK{
	    new_vector_data[i] = vector_data;
	    vector_data = NULL;
	}

	if(vector_data != NULL){
	    ndd_free(&vector_data);
	}	    
    }

    /*  줿ǡѿ˳Ǽ  */
    NDD_IFOK{
	sfd_data->segment_vector = new_vector_data;
	new_vector_data = NULL;
    }

    /*  Իʤɤθդ  */
    if(new_vector_data != NULL){
	for(i = 0 ; i < segment_count; i++){
	    ndd_free(&new_vector_data[i]);
	}
	ndd_free(&new_vector_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_write_to_file(const ndd_sfd_data *sfd_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

    ndd_uint32 segment_count =0;
    NDD_VERIFY_RESULT(ndd_file_header_get_segment_count(header, &segment_count));
 	
    /*  ե졼ǡ񤭹  */
    ndd_uint32 i;
    NDD_FOR(i = 0 ; i < segment_count; i++){
	NDD_VERIFY(fwrite(sfd_data->segment_vector[i], sizeof(ndd_float8), header->segment_dimension, fp) == header->segment_dimension,
		   NDD_ERROR_FILE_WRITE_FAILED);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_print_to_file(const ndd_sfd_data *sfd_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

    ndd_uint32 elements_per_segment = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_segment(header, &elements_per_segment));
    
    /*  ե졼ǡ񤭹  */
    ndd_uint32 i, j, k;
    fputs("SFD data:\n", fp);
    NDD_FOR(i = 0 ; i < header->segment_count; i++){
	fprintf(fp, "  Vector %lu\n", i);
	NDD_FOR(j = 0; j < header->segment_dimension; j += 8){
	    fputs("   ", fp);
	    NDD_FOR(k = 0; k < 8 && j + k < header->segment_dimension; k++){
		fprintf(fp, "%7.4f, ", sfd_data->segment_vector[i][j + k]);
	    }
	    fputs("\n", fp);
	}
	fputs("\n", fp);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfd_data_copy(ndd_sfd_data *dst_sfd_data, const ndd_sfd_data *src_sfd_data, const ndd_file_header *header)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfd_data);
    NDD_VERIFY_POINTER(src_sfd_data);
    NDD_VERIFY_POINTER(header);

    ndd_uint32 segment_count     = 0;
    ndd_uint32 segment_dimension = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_segment_count       (header, &segment_count));
    NDD_IFOK{
	segment_dimension = header->segment_dimension;
    }

    ndd_float8 **new_segment_vector = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_segment_vector, sizeof(ndd_float8 *) * segment_count));

    ndd_uint32 i;
    NDD_FOR(i = 0; i < segment_count; i++){
	ndd_float8 *f = NULL;
	NDD_VERIFY_RESULT(ndd_calloc(&f, sizeof(ndd_float8) * segment_dimension));
	NDD_IFOK{
	    memcpy(f, src_sfd_data->segment_vector[i], sizeof(ndd_float8) * segment_dimension);
	}

	NDD_IFOK{
	    new_segment_vector[i] = f;
	    f = NULL;
	}
	if(f != NULL){  ndd_free(&f);  }
    }

    NDD_IFOK{
	dst_sfd_data->segment_vector = new_segment_vector;
	new_segment_vector = NULL;
    }

    if(new_segment_vector != NULL){
	for(i = 0; i < segment_count; i++){ ndd_free(&new_segment_vector[i]);  }
	ndd_free(&new_segment_vector);
    }

    NDD_RETURN;
}

/*  end of file  */
