/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/sfs.h>
#include <ndd/error.h>
#include <ndd/memory.h>
#include <ndd/file.h>
#include <ndd/ffs.h>
#include <ndd/ffd.h>

#include <string.h>

ndd_result
ndd_sfs_create(ndd_sfs **sfs)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(sfs);

    /*  ΰ  */
    ndd_sfs *new_sfs = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_sfs, sizeof(ndd_sfs)));

    /*  ѿν  */
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_sfs->header));
    NDD_VERIFY_RESULT(ndd_ffs_data_create   (&new_sfs->ffs));
    NDD_VERIFY_RESULT(ndd_sfs_data_create   (&new_sfs->sfs));

    /*  ̤Ǽ  */
    NDD_IFOK{
	*sfs = new_sfs;
	new_sfs = NULL;
    }

    /*  פʥǡդ  */
    if(new_sfs != NULL){
	ndd_sfs_destroy(&new_sfs);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_destroy(ndd_sfs **sfs)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs);

    ndd_sfs *old_sfs;
    NDD_IFOK{
	old_sfs = *sfs;
    }

    NDD_VERIFY_RESULT(ndd_sfs_data_destroy   (&old_sfs->sfs, old_sfs->header));
    NDD_VERIFY_RESULT(ndd_ffs_data_destroy   (&old_sfs->ffs, old_sfs->header));
    NDD_VERIFY_RESULT(ndd_file_header_destroy(&old_sfs->header));
    NDD_VERIFY_RESULT(ndd_free(&old_sfs));
    
    NDD_IFOK{
	*sfs = NULL;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_read_from_file(ndd_sfs *sfs, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs);
    NDD_VERIFY_POINTER(fp);

    /*  إåɤ߹  */
    ndd_file_header *new_header = NULL;
    NDD_VERIFY_RESULT(ndd_file_header_create(&new_header));
    NDD_VERIFY_RESULT(ndd_file_header_read_from_file(new_header, fp));

    /*  FFS ǡɤ߹  */
    ndd_ffs_data *new_ffs_data = NULL;
    NDD_VERIFY_RESULT(ndd_ffs_data_create(&new_ffs_data));
    NDD_VERIFY_RESULT(ndd_ffs_data_read_from_file(new_ffs_data, new_header, fp));

    /*  sfs ǡɤ߹  */
    ndd_sfs_data *new_sfs_data = NULL;
    NDD_VERIFY_RESULT(ndd_sfs_data_create(&new_sfs_data));
    NDD_VERIFY_RESULT(ndd_sfs_data_read_from_file(new_sfs_data, new_header, fp));

    /*  ǡǼ  */
    NDD_IFOK{
	if(sfs->ffs != NULL){
	    NDD_VERIFY_RESULT(ndd_ffs_data_destroy(&sfs->ffs, sfs->header));
	}
	if(sfs->sfs != NULL){
	    NDD_VERIFY_RESULT(ndd_sfs_data_destroy(&sfs->sfs, sfs->header));
	}
	if(sfs->header != NULL){
	    NDD_VERIFY_RESULT(ndd_file_header_destroy(&sfs->header));
	}
    }
    NDD_IFOK{
	sfs->header = new_header;
	sfs->ffs    = new_ffs_data;
	sfs->sfs    = new_sfs_data;
	new_header   = NULL;
	new_ffs_data = NULL;
	new_sfs_data = NULL;
    }

    if(new_ffs_data != NULL){  ndd_ffs_data_destroy   (&new_ffs_data, new_header);  }
    if(new_sfs_data != NULL){  ndd_sfs_data_destroy   (&new_sfs_data, new_header);  }
    if(new_header   != NULL){  ndd_file_header_destroy(&new_header);    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_write_to_file(const ndd_sfs *sfs, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs);
    NDD_VERIFY_POINTER(fp);

    /*  եإå񤭹  */
    NDD_VERIFY_RESULT(ndd_file_header_write_to_file(sfs->header, fp));

    /*  FFS ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_ffs_data_write_to_file(sfs->ffs, sfs->header, fp));

    /*  sfs ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfs_data_write_to_file(sfs->sfs, sfs->header, fp));

    NDD_RETURN;
}

ndd_result
ndd_sfs_print_to_file(const ndd_sfs *sfs, FILE *fp)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs);
    NDD_VERIFY_POINTER(fp);

    /*  եإå񤭹  */
    NDD_VERIFY_RESULT(ndd_file_header_print_to_file(sfs->header, fp));

    /*  FFS ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_ffs_data_print_to_file(sfs->ffs, sfs->header, fp));

    /*  sfs ǡ񤭹  */
    NDD_VERIFY_RESULT(ndd_sfs_data_print_to_file(sfs->sfs, sfs->header, fp));

    NDD_RETURN;
}

ndd_result
ndd_sfs_compare_format(const ndd_sfs *sfs1, const ndd_sfs *sfs2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs1);
    NDD_VERIFY_POINTER(sfs2);
    NDD_VERIFY_POINTER(is_same);
    
    NDD_VERIFY_RESULT(ndd_sfs_header_compare_format(sfs1->header, sfs2->header, is_same));
    NDD_VERIFY_RESULT(ndd_ffs_data_compare(sfs1->ffs, sfs1->header,
					   sfs2->ffs, sfs2->header, is_same));
    
    NDD_RETURN;
}

ndd_result
ndd_sfs_header_compare_format(const ndd_file_header *header1, const ndd_file_header *header2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(header1);
    NDD_VERIFY_POINTER(header2);
    NDD_VERIFY_POINTER(is_same);

    ndd_bool same = false;
    NDD_VERIFY_RESULT(ndd_ffd_header_compare_format(header1, header2, &same));
    NDD_IFOK{
	if(header1->segment_dimension != header2->segment_dimension){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_DIMENSION_MISMATCH);
	}else if(header1->segment_sample_count != header2->segment_sample_count){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_SAMPLE_COUNT_MISMATCH);
	}else if(header1->segment_size != header2->segment_size){
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_SIZE_MISMATCH);
	}else{
	    same = true;
	}
    }

    NDD_IFOK{
	*is_same = same;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_compare(const ndd_sfs_data *sfs_data1, const ndd_file_header *header1,
		     const ndd_sfs_data *sfs_data2, const ndd_file_header *header2, ndd_bool *is_same)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs_data1);
    NDD_VERIFY_POINTER(sfs_data2);
    NDD_VERIFY_POINTER(is_same);

    ndd_bool same = true;
    NDD_VERIFY_RESULT(ndd_sfs_header_compare_format(header1, header2, &same));

    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header1, &elements_per_frame));
    
    ndd_uint32 i;
    NDD_FOR(i = 0; same && i < header1->frame_dimension; i++){
	if(memcmp(sfs_data1->selector_vector[i], sfs_data2->selector_vector[i], sizeof(ndd_float8) * header1->frame_dimension * header1->segment_size) != 0){
	    same = false;
	    NDD_SET_ERROR(NDD_ERROR_SEGMENT_SELECTOR_MISMATCH);
	}
    }

    NDD_IFOK{
	*is_same = same;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_create(ndd_sfs_data **sfs_data)
{ 
    NDD_START;
    NDD_VERIFY_POINTER(sfs_data);

    /*  ΰ  */
    ndd_sfs_data *new_sfs_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_sfs_data, sizeof(ndd_sfs_data)));

    /*  ѿν  */

    /*  ̤Ǽ  */
    NDD_IFOK{
	*sfs_data = new_sfs_data;
	new_sfs_data = NULL;
    }

    /*  פʥǡդ  */
    if(new_sfs_data != NULL){
	ndd_free(&new_sfs_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_destroy(ndd_sfs_data **sfs_data, const ndd_file_header *header)
{
    NDD_START;
    NDD_VERIFY_POINTER(sfs_data);
    NDD_VERIFY_POINTER(header);

    ndd_sfs_data *old_sfs_data;
    NDD_IFOK{
	old_sfs_data = *sfs_data;
    }

    //ndd_uint32 frame_count = 0;
    //NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));

    ndd_uint32 i;
    NDD_FOR(i = 0; i < header->segment_dimension; i++){
	NDD_VERIFY_RESULT(ndd_free(&old_sfs_data->selector_vector[i]));
    }
    NDD_VERIFY_RESULT(ndd_free(&old_sfs_data->selector_vector));
    NDD_VERIFY_RESULT(ndd_free(&old_sfs_data));
    
    NDD_IFOK{
	*sfs_data = NULL;
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_read_from_file(ndd_sfs_data *sfs_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

    /*  ե졼礭ʤɤ  */
    ndd_uint32 frame_count =0;
    NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &frame_count));
    ndd_uint32 elements_per_frame = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_frame(header, &elements_per_frame));
    
       
    	
     /*  ɤ߹ sfs ǡǼΰ  */
    ndd_float8 **new_vector_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_vector_data, sizeof(ndd_float8 *) * header->segment_dimension));

    /*  ƥե졼Υǡɤ߹  */
    ndd_uint32 i;
    NDD_FOR(i = 0 ; i < header->segment_dimension; i++){

	ndd_float8 *vector_data = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&vector_data, sizeof(ndd_float8) * header->frame_dimension*header->segment_size));
	NDD_VERIFY(fread(vector_data, sizeof(ndd_float8), header->frame_dimension*header->segment_size, fp) == header->frame_dimension*header->segment_size,
		   NDD_ERROR_FILE_READ_FAILED);
	
	NDD_IFOK{
	    new_vector_data[i] = vector_data;
	    vector_data = NULL;
	}

	if(vector_data != NULL){
	    ndd_free(&vector_data);
	}	    
    }

    /*  줿ǡѿ˳Ǽ  */
    NDD_IFOK{
	if(sfs_data->selector_vector != NULL){
	   // ndd_uint32 old_frame_count =0;
	   // NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &old_frame_count));
	    NDD_FOR(i = 0 ; i < header->segment_dimension; i++){
		NDD_VERIFY_RESULT(ndd_free(&sfs_data->selector_vector[i]));
	    }	
	    NDD_VERIFY_RESULT(ndd_free(&sfs_data->selector_vector));
	}
    }
    NDD_IFOK{
	sfs_data->selector_vector = new_vector_data;
	new_vector_data = NULL;
    }

    /*  Իʤɤθդ  */
    if(new_vector_data != NULL){
	for(i = 0 ; i < header->segment_dimension; i++){
	    ndd_free(&new_vector_data[i]);
	}
	ndd_free(&new_vector_data);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_write_to_file(const ndd_sfs_data *sfs_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

 //   ndd_uint32 frame_count = 0;
 //   NDD_VERIFY_RESULT(ndd_file_header_get_frame_count(header, &header->segment_dimension);
 	
    /*  ե졼ǡ񤭹  */
    ndd_uint32 i;
    NDD_FOR(i = 0 ; i <header->segment_dimension;  i++){
	NDD_VERIFY(fwrite(sfs_data->selector_vector[i], sizeof(ndd_float8),
			  header->frame_dimension*header->segment_size, fp) == header->frame_dimension*header->segment_size,
		   NDD_ERROR_FILE_WRITE_FAILED);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_print_to_file(const ndd_sfs_data *sfs_data, const ndd_file_header *header, FILE *fp)
{
    NDD_START;

    ndd_uint32 elements_per_segment = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_segment(header, &elements_per_segment));
    
    /*  ե졼ǡ񤭹  */
    ndd_uint32 i, j, k;
    fputs("SFS data:\n", fp);
    NDD_FOR(i = 0 ; i < header->segment_dimension; i++){
	fprintf(fp, "  Vector %lu\n", i);
	NDD_FOR(j = 0; j < elements_per_segment; j += 8){
	    fputs("   ", fp);
	    NDD_FOR(k = 0; k < 8 && j + k < elements_per_segment; k++){
		fprintf(fp, "%7.4f, ", sfs_data->selector_vector[i][j + k]);
	    }
	    fputs("\n", fp);
	}
	fputs("\n", fp);
    }

    NDD_RETURN;
}

ndd_result
ndd_sfs_data_copy(ndd_sfs_data *dst_sfs_data, const ndd_sfs_data *src_sfs_data, const ndd_file_header *header)
{
    NDD_START;
    NDD_VERIFY_POINTER(dst_sfs_data);
    NDD_VERIFY_POINTER(src_sfs_data);
    NDD_VERIFY_POINTER(header);

    ndd_uint32 elements_per_segment = 0;
    NDD_VERIFY_RESULT(ndd_file_header_get_elements_per_segment(header, &elements_per_segment));

    ndd_float8 **new_selector_vectors      = NULL;
    ndd_uint32   new_selector_vector_count = 0;

    ndd_uint32 i;
    NDD_FOR(i = 0; i < header->segment_dimension; i++){
	ndd_float8 *new_selector_vector = NULL;
	NDD_VERIFY_RESULT(ndd_malloc(&new_selector_vector, sizeof(ndd_float8) * elements_per_segment));
	NDD_IFOK{
	    memcpy(new_selector_vector, src_sfs_data->selector_vector[i],sizeof(ndd_float8) * elements_per_segment);
	}

	NDD_VERIFY_RESULT(ndd_realloc(&new_selector_vectors, sizeof(ndd_float8 *) * (new_selector_vector_count + 1)));
	NDD_IFOK{
	    new_selector_vectors[new_selector_vector_count++] = new_selector_vector;
	    new_selector_vector = NULL;
	}

	if(new_selector_vector != NULL){  ndd_free(&new_selector_vector);  }
    }

    NDD_IFOK{
	dst_sfs_data->selector_vector = new_selector_vectors;
	new_selector_vectors = NULL;
    }

    if(new_selector_vectors != NULL){
	for(i = 0; i < new_selector_vector_count; i++){
	    ndd_free(&new_selector_vectors[i]);
	}
	ndd_free(&new_selector_vectors);
    }

    NDD_RETURN;
}

/*  end of file  */
