/*
    Copyright 2006-2009 Nagoya University

    This file is part of NDD (Near Duplicate Detector).

    NDD is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    NDD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with NDD.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <ndd/vector.h>
#include <ndd/error.h>
#include <ndd/memory.h>

/*    */

ndd_result
ndd_vector_create(ndd_vector **vector, ndd_uint32 count)
{
    NDD_START;
    NDD_VERIFY_POINTER(vector);

    ndd_vector *new_vector      = NULL;
    ndd_float8 *new_vector_data = NULL;
    NDD_VERIFY_RESULT(ndd_calloc(&new_vector, sizeof(ndd_vector)));
    NDD_VERIFY_RESULT(ndd_calloc(&new_vector_data, sizeof(ndd_float8) * count));
    NDD_IFOK{
	new_vector->data  = new_vector_data;
	new_vector->count = count;
	*vector = new_vector;
	new_vector      = NULL;
	new_vector_data = NULL;
    }

    if(new_vector_data != NULL){
	ndd_free(&new_vector_data);
    }
    if(new_vector != NULL){
	ndd_free(&new_vector);
    }

    NDD_RETURN;
}

ndd_result
ndd_vector_destroy(ndd_vector **vector)
{
    NDD_START;
    NDD_VERIFY_POINTER(vector);

    NDD_VERIFY_RESULT(ndd_free(&(*vector)->data));
    NDD_VERIFY_RESULT(ndd_free(vector));
    
    NDD_RETURN;
}

/*  end of file  */
